/*	Copyright_mac.cpp
 *
 *		This is the module which handles throwing up the 'about'
 *	dialog box.
 *
 *		This handles both putting up the dialog box when the user
 *	requests it, and when the application first starts.
 */


/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <Gestalt.h>
#include <XDataUtil.h>
#include <XError.h>
#include "Copyright_mac.h"

/************************************************************************/
/*																		*/
/*	Constants		 													*/
/*																		*/
/************************************************************************/

#define KDialogAboutColor	129
#define KDialogAboutBW		128

/************************************************************************/
/*																		*/
/*	Globals			 													*/
/*																		*/
/************************************************************************/

static DialogPtr	GStartDialog;		// Start dialog (use LastClickTime)
static long  		GDelay;				// Delay to show dialog box
static short 		GRedraw = 0;		// Set if need to draw version

/************************************************************************/
/*																		*/
/*	Internal Routines 													*/
/*																		*/
/************************************************************************/

/*	DrawVersionString
 *
 *		This function (which can be overridden by the user) draws the
 *  version if the version resource exists
 */

static void DrawVersionString(Rect *r)
{
	FontInfo finfo;
	unsigned char buffer[64];
	
    struct VersRecord {
		unsigned char	majVersion;
		unsigned char	minVersion;
		unsigned char	devel;
		unsigned char	release;
		short			region;
		unsigned char	shortVers[2];
	} **vers;
	short x,y;
	
	if (NULL == (vers = (VersRecord **)GetResource('vers',1))) return;
	HLock((Handle)vers);
	
	/*
	 *  Print the version string (the pascal string contained in this thing)
	 */
	
	pstrcpy(buffer,(**vers).shortVers);
#	ifdef powerc
		pstrcat(buffer,"\p(ppc)");
#	else
		pstrcat(buffer,"\p(68k)");
#	endif

	TextFont(4);
	TextSize(9);
	TextMode(srcOr);
	GetFontInfo(&finfo);
	y = r->top + finfo.ascent;
	x = r->right - StringWidth(buffer);
	MoveTo(x,y);
	DrawString(buffer);
	
	HUnlock((Handle)vers);
	ReleaseResource((Handle)vers);
	
	TextFont(0);
	TextSize(12);
}

/*	AboutMeFilter
 *
 *		This is the filter procedure for simply waiting around until the
 *	user does something.
 */

static pascal Boolean AboutMeFilter(DialogPtr dlog, EventRecord *event, short *item)
{
	if (GRedraw) {
		SetPort(dlog);
		DrawVersionString(&(dlog->portRect));
		GRedraw = 0;
	}
	
	if ((event->what == mouseDown) || (event->what == keyDown)) {
		*item = 1;
		return 1;
	} else if (event->what == updateEvt) {
		GRedraw = 1;
		return 0;
	} else return 0;
}

/*	OpenAboutMe
 *
 *		Open the about me dialog box
 */

static DialogPtr OpenAboutMe(void)
{
	DialogPtr dlog;
	GrafPtr port;
	Rect r;
	short x,y;
	long version;
	
	Gestalt(gestaltQuickdrawVersion,&version);
	
	if (version >= 0x100) {
		dlog = GetNewDialog(KDialogAboutColor,NULL,(WindowPtr)-1);
		if (dlog == NULL) {
			dlog = GetNewDialog(KDialogAboutBW,NULL,(WindowPtr)-1);
			if (dlog == NULL) {
				throw XPostError("About Dialog Resource Missing");
				//MissingResource('DLOG',KDialogAboutBW);
			}
		}
	} else {
		dlog = GetNewDialog(KDialogAboutBW,NULL,(WindowPtr)-1);
		if (dlog == NULL) {
			throw XPostError("About Dialog Resource Missing");
			//MissingResource('DLOG',KDialogAboutBW);
		}
	}
	
	GetWMgrPort(&port);						// Center the window
	r = port->portRect;
	x = r.right - r.left;
	y = r.bottom - r.top;
	r = dlog->portRect;
	x -= r.right - r.left;
	y -= r.bottom - r.top;
	x /= 2;
	y /= 2;
	MoveWindow(dlog,x,y,0);
	
	ShowWindow(dlog);
	SetPort(dlog);
	
	return dlog;
}

/************************************************************************/
/*																		*/
/*	Interface Routines 													*/
/*																		*/
/************************************************************************/

/*	ShowMacCopyright
 *
 *		how to run a dialog box
 */

void ShowMacCopyright(void)
{
	short i;
	ModalFilterUPP proc;
	
	DialogPtr dlog = OpenAboutMe();
	if (!dlog) return;
	proc = (ModalFilterUPP)NewRoutineDescriptor((ProcPtr)AboutMeFilter,uppModalFilterProcInfo,GetCurrentISA());
	
	for (;;) {
		ModalDialog(proc,&i);
		if (i == 1) break;
	}

	DisposeRoutineDescriptor(proc);
	DisposeDialog(dlog);
}

/*	OpenMacCopyright
 *
 *		Open the temporary about me dialog box and do the update
 *	stuff
 */

void OpenMacCopyright(void)
{
	GStartDialog = OpenAboutMe();
	
	SetPort(GStartDialog);
	DrawDialog(GStartDialog);
	DrawVersionString(&(GStartDialog->portRect));	// Redraw version str
	
	GDelay = TickCount() + 120;						// 2 second minimum
}

/*	CloseMacCopyright
 *
 *		Close the temporary about me
 */

void CloseMacCopyright(void)
{
	if (GStartDialog) {
		while (TickCount() < GDelay) ;
		DisposeDialog(GStartDialog);
		GStartDialog = NULL;
	}
}
