/*	WindowProc.cpp
 *
 *		This contains the four callback procs that are used to make my
 *	windows work
 */

#include <XDataUtil.h>
#include <XError.h>
#include <XWindow.h>
#include <XApplication.h>

/************************************************************************/
/*																		*/
/*	Keyboard support													*/
/*																		*/
/************************************************************************/

/*	GetExtendedKeyCode
 *
 *		This returns the extended key code for the KeyCommand function.
 *	If the key event is not an extended key code key, this returns -1
 */

static short GetExtendedKeyCode(WPARAM wp, LPARAM lp)
{
	if (lp & (1 << 24)) {
		switch (wp) {
			case VK_UP:
				return KKeyUpCursor;
			case VK_DOWN:
				return KKeyDownCursor;
			case VK_LEFT:
				return KKeyLeftCursor;
			case VK_RIGHT:
				return KKeyRightCursor;
			case VK_INSERT:
				return KKeyInsert;
			case VK_DELETE:
				return KKeyDelete;
			case VK_HOME:
				return KKeyHome;
			case VK_END:
				return KKeyEnd;
			case VK_PRIOR:
				return KKeyPageUp;
			case VK_NEXT:
				return KKeyPageDown;
			case VK_ESCAPE:
				return KKeyEscape;
			case VK_HELP:
				return KKeyHelp;
			case VK_NUMLOCK:
				return KKeyNumLock;
			case VK_F1:
				return KKeyF1;
			case VK_F2:
				return KKeyF2;
			case VK_F3:
				return KKeyF3;
			case VK_F4:
				return KKeyF4;
			case VK_F5:
				return KKeyF5;
			case VK_F6:
				return KKeyF6;
			case VK_F7:
				return KKeyF7;
			case VK_F8:
				return KKeyF8;
			case VK_F9:
				return KKeyF9;
			case VK_F10:
				return KKeyF10;
			case VK_F11:
				return KKeyF11;
			case VK_F12:
				return KKeyF12;
			case VK_F13:
				return KKeyF13;
			case VK_F14:
				return KKeyF14;
			case VK_F15:
				return KKeyF15;
			default:
				return -1;
		}
	} else {
		return -1;
	}
}

/************************************************************************/
/*																		*/
/*	MDI framework callbacks												*/
/*																		*/
/************************************************************************/

/*	_GMainMDIWindowProc
 *
 *		How to deal with a frame about this thing
 */

LRESULT CALLBACK _GMainMDIWindowProc(HWND w, UINT msg, WPARAM wp, LPARAM lp)
{
	RECT r;
//	RECT s;
//	RECT t;
	CLIENTCREATESTRUCT ccs;
//	PAINTSTRUCT ps;
//	HDWP deferHandle;
	short flags;
	HMENU menu;
	short item;
//	HICON icon;
//	short oldMode;
	short mods;
	short c;
	
	switch (msg) {
		/*
		 *  Construction of this window: construct the children
		 *	window inside of this thing
		 */

		case WM_CREATE:
			ccs.hWindowMenu = NULL;
			ccs.idFirstChild = 10000;				// Very high number
			_GMDIWindow = CreateWindow("MDIClient","",
							WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
							0,0,0,0,w,NULL,_GInstance,(LPSTR)&ccs);
			if (_GMDIWindow == NULL) return -1;		// Failure to do so
			ShowWindow(_GMDIWindow,SW_SHOW);
			return 0;
		
		/*
		 *  Quit event
		 */
		
		case WM_DESTROY:
			_GMainWindow = NULL;
			PostQuitMessage(0);
			return 0;
			
		/*
		 *	Close main application event
		 */
		
		case WM_CLOSE:
			_GApp->Quit();							// Process quit message
			return 0;
		
		/*
		 *  Resize events
		 */
		
		case WM_SIZE:
			GetClientRect(w,&r);
			MoveWindow(_GMDIWindow,r.left,r.top,r.right-r.left,
							r.bottom-r.top,TRUE);
			return 0;
		
		/*
		 *  Process menu highlights
		 */
		
		case WM_MENUSELECT:
			flags = HIWORD(wp);
			item  = LOWORD(wp);
			menu  = (HMENU)lp;
			
			/*
			 *  Get the string and put it in the help box
			 */
			
//			if ((menu == NULL) && (flags == -1)) {
//				SetHelpContent(NULL);
//			} else {
//				if (flags & MF_POPUP) SetHelpContent(NULL);
//				else {
//					char buffer[256];
//					
//					if (LoadString(_GInstance,item,buffer,sizeof(buffer))) {
//						SetHelpContent(buffer);
//					} else {
//						SetHelpContent(NULL);
//					}
//				}
//			}
			return 0;

		/*
		 *  Process menu/control event
		 */
		
		case WM_COMMAND:
			item = LOWORD(wp);
			if (_GApp->_GetCurFocus()->SendDispatch(KEventDoMenuCommand,item,NULL)) return 0;
			break;
		
		/*
		 *	Process WM_PAINT message to draw the small icon when this
		 *	application has been iconic
		 */
		
//		case WM_PAINT:
//			BeginPaint(w,&ps);
//			if (IsIconic(w)) {
//				icon = LoadIcon(_GInstance,MAKEINTRESOURCE(_GApp->GetIconID()));
//				if (icon) {
//					oldMode = GetMapMode(ps.hdc);
//					SetMapMode(ps.hdc,MM_TEXT);
//					DrawIcon(ps.hdc,0,0,icon);
//					SetMapMode(ps.hdc,oldMode);
//					DestroyIcon(icon);
//				}
//			}
//			EndPaint(w,&ps);
//			return 0;
		/*
		 *  Handle key down. Figure out the appropriate modifiers and
		 *	dispatch the key event. This also handles conversion of the
		 *	various virtual key codes into Macintosh-equivalent key codes
		 */
		
		case WM_KEYDOWN:
			mods = 0;
			if (GetKeyState(VK_SHIFT) & (~1)) mods |= KKeyShift;
			if (GetKeyState(VK_CONTROL) & (~1)) mods |= KKeyControl;
			if (GetKeyState(VK_MENU) & (~1)) mods |= KKeyMenu;
			if (GetKeyState(VK_LBUTTON) & (~1)) mods |= KKeyButton;
			if (GetKeyState(VK_MBUTTON) & (~1)) mods |= KKeyMButton;
			if (GetKeyState(VK_RBUTTON) & (~1)) mods |= KKeyRButton;
			
			c = GetExtendedKeyCode(wp,lp);
			if (c != -1) {
				_GApp->_GetCurFocus()->SendDispatch(KEventKey,MAKEDWORD(mods | KSpecialKey,c),NULL);
				return 0;
			}
			break;
			
		case WM_CHAR:
			mods = 0;
			if (GetKeyState(VK_SHIFT) & (~1)) mods |= KKeyShift;
			if (GetKeyState(VK_CONTROL) & (~1)) mods |= KKeyControl;
			if (GetKeyState(VK_MENU) & (~1)) mods |= KKeyMenu;
			if (GetKeyState(VK_LBUTTON) & (~1)) mods |= KKeyButton;
			if (GetKeyState(VK_MBUTTON) & (~1)) mods |= KKeyMButton;
			if (GetKeyState(VK_RBUTTON) & (~1)) mods |= KKeyRButton;
			
			c = GetExtendedKeyCode(wp,lp);
			if (c == -1) {
				_GApp->_GetCurFocus()->SendDispatch(KEventKey,MAKEDWORD(mods,c),NULL);
				return 0;
			}
			break;
				
	}
	
	return DefFrameProc(w,_GMDIWindow,msg,wp,lp);
}

/*	_GChildWindowProc
 *
 *		The window procedure for XGWindow-based procedures
 */

LRESULT CALLBACK _GChildWindowProc(HWND w, UINT msg, WPARAM wp, LPARAM lp)
{
	int defFlag = 0;
	Point pt;
    MINMAXINFO *mm;
	XGWindow *tw;
	static short gMods;
	long ret = 0;
//	HPALETTE oldPalette;
//	HPALETTE newPalette;
	PAINTSTRUCT ps;
//	HICON icon;
//	int oldMode;
	HWND control;
	short flag;
	
	/*
	 *	Trap errors
	 */
	
	try {
		
		/*
		 *  Default (used for most things) -- initialize the parameters
		 *	for this window to dispatch events
		 */
		
		tw = XGWindow::_GetXGWindow(w);
		if (tw == NULL) tw = _GWindow;
		
//		if (tw) flag = (tw->_Attributes() & KAttrNoBound) ? 1 : 0;
//		else flag = 0;
		flag = 0;
			
		/*
		 *	Handle the events which are processed during construction or during
		 *	creation.
		 */
		
		switch (msg) {
			/*
			 *  Construction management -- create this window, or at least
			 *	do my half of this thing
			 */
			
			case WM_CREATE:
				tw = _GWindow;						// Initialize this thing
				SetProp(w,_GWindowClass,(HANDLE)tw);
				tw->fViewHWND = w;					// Fix up back pointer too
				break;
			
			/*
			 *  Destruction -- handle destroying this thing
			 */
			
			case WM_DESTROY:
				if (tw) {
//					if (tw->_Attributes() & KAttrNoBound) {
//						// Wipe out top window attribute if this thing is.
//						_GApp->SetTopWindow(NULL,0);
//					}
					tw->fViewHWND = NULL;				// If this is still valid,
					delete tw;						// then we have to do this.
				}
				break;
			
			/*
			 *  Minimum/maximum stuff: determine minimum, maximum sizes
			 */
			
			case WM_GETMINMAXINFO:
				mm = (MINMAXINFO *)lp;
				if (tw) {
					pt = tw->MinWindowSize();
					if (pt.h != -1) mm->ptMinTrackSize.x = pt.h;
					if (pt.v != -1) mm->ptMinTrackSize.y = pt.v;
					pt = tw->MaxWindowSize();
					if (pt.h != -1) mm->ptMaxTrackSize.x = pt.h;
					if (pt.v != -1) mm->ptMaxTrackSize.y = pt.v;
					pt = tw->ZoomWindowSize();
					if (pt.h != -1) mm->ptMaxSize.x = pt.h;
					if (pt.v != -1) mm->ptMaxSize.y = pt.v;
				} else {
					mm->ptMinTrackSize.x = 150;
					mm->ptMinTrackSize.y = 100;
				}
				break;
	
			default:
				break;
		}
		
		/*
		 *  Dispatch the events which are handled only after window construction.
		 */
		
		switch (msg) {
			
			/*
			 *  Close: Handle the closing of this window. This will call
			 *	the appropriate destructors
			 */
			
			case WM_CLOSE:
				tw->CloseWindow();
				break;
				
			/*
			 *  Erase background stuff: If I'm iconic, do the right thing
			 */
			
		    case WM_ERASEBKGND:
		    	if (!flag) {
					if (IsIconic(w)) return DefMDIChildProc(w,WM_ICONERASEBKGND,wp,lp);
					else return DefMDIChildProc(w,msg,wp,lp);
				} else {
					if (IsIconic(w)) return DefWindowProc(w,WM_ICONERASEBKGND,wp,lp);
					else return DefWindowProc(w,msg,wp,lp);
				}
			
			/*
			 *  Handle activation routines for dealing with activating/
			 *	deactivating windows
			 */
			
			case WM_MDIACTIVATE:
				if (w == (HWND)lp) {
					if (tw && tw->fViewHWND) tw->DoActivate(1);
				} else {
					if (tw && tw->fViewHWND) tw->DoActivate(0);
				}
				break;
				
			/*
			 *  Handle resize of this thing.
			 */
			
			case WM_SIZE:
				defFlag = 1;
				
				if (tw != _GWindow) {
					if (tw && (tw->fViewHWND)) {
						if (!tw->IsIconic()) {
							tw->_ProcessSizeEvent();
						} else {
//###						tw->Iconic();
						}
					}
				}
				break;
				
			/*
			 *  Handle button processing
			 */
			
			case WM_LBUTTONDOWN:
			case WM_LBUTTONDBLCLK:
			case WM_RBUTTONDOWN:
			case WM_RBUTTONDBLCLK:
			case WM_MBUTTONDOWN:
			case WM_MBUTTONDBLCLK:
				if (w == GetCapture()) break;
				
				gMods = 0;
				if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
				if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
				if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
				if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
				if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
				if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				
				if ((msg == WM_LBUTTONDBLCLK) || (msg == WM_MBUTTONDBLCLK) ||
						(msg == WM_RBUTTONDBLCLK))
					gMods |= KDoubleClick;
	
				if (tw) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					if (tw->DoMouseDown(pt,gMods)) {
						SetCapture(w);
					}
				}
				break;
				
			case WM_MOUSEMOVE:
				if (w == GetCapture()) {
					gMods |= KCapturedMouse;
				} else {
					gMods = 0;
					if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
					if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
					if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
					if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
					if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
					if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				}
				if (tw) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					tw->DoMouseMove(pt,gMods);
				}
				break;
				
			case WM_LBUTTONUP:
			case WM_RBUTTONUP:
			case WM_MBUTTONUP:
				if (w == GetCapture()) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					tw->DoMouseUp(pt,gMods);
					ReleaseCapture();
				}
				break;
				
			/*
			 *  Handle updating the display
			 */
			
			case WM_PAINT:
				/*
				 *  Paint this window's icon
				 */
				
				if (IsIconic(w)) {
//					BeginPaint(w,&ps);
//					
//					/*
//					 *  Paint the icon
//					 */
//					
//					icon = LoadWindowIcon(tw);
//					if (icon) {
//						oldMode = GetMapMode(ps.hdc);
//						SetMapMode(ps.hdc,MM_TEXT);
//						DrawIcon(ps.hdc,0,0,icon);
//						SetMapMode(ps.hdc,oldMode);
//						DestroyIcon(icon);
//					}
//					EndPaint(w,&ps);
				} else {
					/*
					 *  Draw the contents of this thing
					 */
					
					BeginPaint(w,&ps);
					
					if (tw) {
//						newPalette = tw->GetWindowPalette();
//						if (newPalette) {
//							oldPalette = SelectPalette(ps.hdc,newPalette,FALSE);
//							RealizePalette(ps.hdc);
//							tw->ProcessUpdateEvent(ps.rcPaint);
//							SelectPalette(ps.hdc,oldPalette,FALSE);
//							DeleteObject(newPalette);
//						} else {
							tw->DoDrawView(ps.rcPaint);
//						}
					}
					EndPaint(w,&ps);
				}
				break;
			
			/*
			 *  Get the drag icon for this window when the user moves the
			 *	iconic representation around
			 */
				
//			case WM_QUERYDRAGICON:
//				ret = (long)LoadWindowIcon(tw);
//				break;
			
			/*
			 *  Color Palette Support
			 */
			
//			case WM_PALETTECHANGED:
//				if ((HWND)wp == w) break;
//				
//				/* Fall through to */
//			case WM_QUERYNEWPALETTE:
//				if (tw == NULL) break;
//				
//				ps.hdc = GetDC(w);
//				newPalette = tw->GetWindowPalette();
//				if (newPalette) {
//					oldPalette = SelectPalette(ps.hdc,newPalette,FALSE);
//					ret = RealizePalette(ps.hdc);
//					SelectPalette(ps.hdc,oldPalette,FALSE);
//					ReleaseDC(w,ps.hdc);
//					DeleteObject(newPalette);
//					if (ret > 0) InvalidateRect(w,NULL,TRUE);
//				}
//				break;
			
			/*
			 *  Scroll bar processing
			 */
			
			case WM_COMMAND:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				}
				break;
				
			case WM_VSCROLL:
			case WM_HSCROLL:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				}
				break;
				
			/*
			 *  Default processing
			 */
			
			default:
				defFlag = 1;
				break;
		}
	
		if (!flag) {
			if (defFlag) return DefMDIChildProc(w,msg,wp,lp);
			else return ret;
		} else {
			if (defFlag) return DefWindowProc(w,msg,wp,lp);
			else return ret;
		}
	}
	catch (XError &e) {
		e.DisplayError();
	}
	return 0;
}

/************************************************************************/
/*																		*/
/*	Single Window App callback											*/
/*																		*/
/************************************************************************/

/*	_GSingleWindowProc
 *
 *		This is the callback proc for a single window application. This
 *	is a lot like a cross between the multiwindow frame and multiwindow
 *	MDI child
 */

LRESULT CALLBACK _GSingleWindowProc(HWND w, UINT msg, WPARAM wp, LPARAM lp)
{
	static short gMods;
	XGWindow *tw;
	Point pt;
    MINMAXINFO *mm;
    PAINTSTRUCT ps;
    HWND control;
	HWND menu;
	short flags;
	short item;
    int defFlag = 0;
	long ret = 0;
	
	/*
	 *	Trap errors
	 */
	
	try {
		
		/*
		 *  Default (used for most things) -- initialize the parameters
		 *	for this window to dispatch events
		 */
		
		tw = XGWindow::_GetXGWindow(w);
		if (tw == NULL) tw = _GWindow;
		
//		if (tw) flag = (tw->_Attributes() & KAttrNoBound) ? 1 : 0;
//		else flag = 0;
			
		/*
		 *	Handle the events which are processed during construction or during
		 *	creation.
		 */
		
		switch (msg) {
			/*
			 *  Construction management -- create this window, or at least
			 *	do my half of this thing
			 */
			
			case WM_CREATE:
				tw = _GWindow;						// Initialize this thing
				SetProp(w,_GWindowClass,(HANDLE)tw);
				tw->fViewHWND = w;					// Fix up back pointer too
				break;
			
			/*
			 *  Destruction -- handle destroying this thing
			 */
			
			case WM_DESTROY:
				if (tw) {
//					if (tw->_Attributes() & KAttrNoBound) {
//						// Wipe out top window attribute if this thing is.
//						_GApp->SetTopWindow(NULL,0);
//					}
					tw->fViewHWND = NULL;				// If this is still valid,
					delete tw;						// then we have to do this.
				}
				break;

			/*
			 *  Minimum/maximum stuff: determine minimum, maximum sizes
			 */
			
			case WM_GETMINMAXINFO:
				mm = (MINMAXINFO *)lp;
				if (tw) {
					pt = tw->MinWindowSize();
					if (pt.h != -1) mm->ptMinTrackSize.x = pt.h;
					if (pt.v != -1) mm->ptMinTrackSize.y = pt.v;
					pt = tw->MaxWindowSize();
					if (pt.h != -1) mm->ptMaxTrackSize.x = pt.h;
					if (pt.v != -1) mm->ptMaxTrackSize.y = pt.v;
					pt = tw->ZoomWindowSize();
					if (pt.h != -1) mm->ptMaxSize.x = pt.h;
					if (pt.v != -1) mm->ptMaxSize.y = pt.v;
				} else {
					mm->ptMinTrackSize.x = 150;
					mm->ptMinTrackSize.y = 100;
				}
				break;
	
			default:
				break;
		}

		
		/*
		 *  Dispatch the events which are handled only after window construction.
		 */
		
		switch (msg) {
			
			/*
			 *  Close: Handle the closing of this window. This will call
			 *	the appropriate destructors
			 */
			
			case WM_CLOSE:
				tw->CloseWindow();
				break;
				
			/*
			 *  Erase background stuff: If I'm iconic, do the right thing
			 */
			
		    case WM_ERASEBKGND:
				if (IsIconic(w)) {
					return DefWindowProc(w,WM_ICONERASEBKGND,wp,lp);
				} else {
					return DefWindowProc(w,msg,wp,lp);
				}
			
			/*
			 *  Handle activation routines for dealing with activating/
			 *	deactivating windows
			 */
			
			case WM_ACTIVATE:
				if (LOWORD(wp) != WA_INACTIVE) {
					if (tw && tw->fViewHWND) tw->DoActivate(1);
				} else {
					if (tw && tw->fViewHWND) tw->DoActivate(0);
				}
				break;
				
			/*
			 *  Handle resize of this thing.
			 */
			
			case WM_SIZE:
				defFlag = 1;
				
				if (tw != _GWindow) {
					if (tw && (tw->fViewHWND)) {
						if (!tw->IsIconic()) {
							tw->DoSizeView();
						} else {
//###						tw->Iconic();
						}
					}
				}
				break;
				
			/*
			 *  Handle button processing
			 */
			
			case WM_LBUTTONDOWN:
			case WM_LBUTTONDBLCLK:
			case WM_RBUTTONDOWN:
			case WM_RBUTTONDBLCLK:
			case WM_MBUTTONDOWN:
			case WM_MBUTTONDBLCLK:
				if (w == GetCapture()) break;
				
				gMods = 0;
				if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
				if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
				if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
				if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
				if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
				if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				
				if ((msg == WM_LBUTTONDBLCLK) || (msg == WM_MBUTTONDBLCLK) ||
						(msg == WM_RBUTTONDBLCLK))
					gMods |= KDoubleClick;
	
				if (tw) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					if (tw->DoMouseDown(pt,gMods)) {
						SetCapture(w);
					}
				}
				break;
				
			case WM_MOUSEMOVE:
				if (w == GetCapture()) {
					gMods |= KCapturedMouse;
				} else {
					gMods = 0;
					if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
					if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
					if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
					if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
					if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
					if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				}
				if (tw) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					tw->DoMouseMove(pt,gMods);
				}
				break;
				
			case WM_LBUTTONUP:
			case WM_RBUTTONUP:
			case WM_MBUTTONUP:
				if (w == GetCapture()) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					tw->DoMouseUp(pt,gMods);
					ReleaseCapture();
				}
				break;
				
			/*
			 *  Handle updating the display
			 */
			
			case WM_PAINT:
				/*
				 *  Paint this window's icon
				 */
				
				if (IsIconic(w)) {
//					BeginPaint(w,&ps);
//					
//					/*
//					 *  Paint the icon
//					 */
//					
//					icon = LoadWindowIcon(tw);
//					if (icon) {
//						oldMode = GetMapMode(ps.hdc);
//						SetMapMode(ps.hdc,MM_TEXT);
//						DrawIcon(ps.hdc,0,0,icon);
//						SetMapMode(ps.hdc,oldMode);
//						DestroyIcon(icon);
//					}
//					EndPaint(w,&ps);
				} else {
					/*
					 *  Draw the contents of this thing
					 */
					
					BeginPaint(w,&ps);
					
					if (tw) {
//						newPalette = tw->GetWindowPalette();
//						if (newPalette) {
//							oldPalette = SelectPalette(ps.hdc,newPalette,FALSE);
//							RealizePalette(ps.hdc);
//							tw->ProcessUpdateEvent(ps.rcPaint);
//							SelectPalette(ps.hdc,oldPalette,FALSE);
//							DeleteObject(newPalette);
//						} else {
							tw->DoDrawView(ps.rcPaint);
//						}
					}
					EndPaint(w,&ps);
				}
				break;
			
			/*
			 *  Get the drag icon for this window when the user moves the
			 *	iconic representation around
			 */
				
//			case WM_QUERYDRAGICON:
//				ret = (long)LoadWindowIcon(tw);
//				break;
			
			/*
			 *  Color Palette Support
			 */
			
//			case WM_PALETTECHANGED:
//				if ((HWND)wp == w) break;
//				
//				/* Fall through to */
//			case WM_QUERYNEWPALETTE:
//				if (tw == NULL) break;
//				
//				ps.hdc = GetDC(w);
//				newPalette = tw->GetWindowPalette();
//				if (newPalette) {
//					oldPalette = SelectPalette(ps.hdc,newPalette,FALSE);
//					ret = RealizePalette(ps.hdc);
//					SelectPalette(ps.hdc,oldPalette,FALSE);
//					ReleaseDC(w,ps.hdc);
//					DeleteObject(newPalette);
//					if (ret > 0) InvalidateRect(w,NULL,TRUE);
//				}
//				break;
			
			/*
			 *  Scroll bar processing
			 */
			
			case WM_COMMAND:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				} else {
					// Menu dispatch
					if (_GApp->_GetCurFocus()->SendDispatch(KEventDoMenuCommand,LOWORD(wp),NULL)) return 0;
				}
				break;
				
			case WM_VSCROLL:
			case WM_HSCROLL:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				}
				break;
				
			/*
			 *	Menu hilites
			 */
				
		case WM_MENUSELECT:
			flags = HIWORD(wp);
			item  = LOWORD(wp);
			menu  = (HMENU)lp;
			
			/*
			 *  Get the string and put it in the help box
			 */
			
//			if ((menu == NULL) && (flags == -1)) {
//				SetHelpContent(NULL);
//			} else {
//				if (flags & MF_POPUP) SetHelpContent(NULL);
//				else {
//					char buffer[256];
//					
//					if (LoadString(_GInstance,item,buffer,sizeof(buffer))) {
//						SetHelpContent(buffer);
//					} else {
//						SetHelpContent(NULL);
//					}
//				}
//			}
			return 0;

			/*
			 *  Default processing
			 */
			
			default:
				defFlag = 1;
				break;
		}
	
		if (defFlag) return DefWindowProc(w,msg,wp,lp);
		else return ret;
	}
	catch (XError &e) {
		e.DisplayError();
	}
	return 0;
}

/************************************************************************/
/*																		*/
/*	View Callback														*/
/*																		*/
/************************************************************************/

/*	_GViewWindowProc
 *
 *		This is the callback proc for a view.
 */

LRESULT CALLBACK _GViewWindowProc(HWND w, UINT msg, WPARAM wp, LPARAM lp)
{
	XGView *v;
	short gMods;
	PAINTSTRUCT ps;
	Point pt;
	HWND control;
    int defFlag = 0;
	long ret = 0;
		
	/*
	 *	Trap errors
	 */
	
	try {
		
		/*
		 *  Default (used for most things) -- initialize the parameters
		 *	for this window to dispatch events
		 */
		
		v = XGWindow::_GetXGView(w);
		if (v == NULL) v = _GView;
			
		/*
		 *	Handle the events which are processed during construction or during
		 *	creation.
		 */
		
		switch (msg) {
			/*
			 *  Construction management -- create this window, or at least
			 *	do my half of this thing
			 */
			
			case WM_CREATE:
				v = _GView;							// Initialize this thing
				SetProp(w,_GViewClass,(HANDLE)v);
				v->fViewHWND = w;					// Fix up back pointer too
				break;
			
			/*
			 *  Destruction -- handle destroying this thing
			 */
			
			case WM_DESTROY:
				if (v) {
					v->fViewHWND = NULL;			// If this is still valid,
					delete v;						// then we have to do this.
				}
				break;
			
			default:
				break;
		}

		
		/*
		 *  Dispatch the events which are handled only after window construction.
		 */
		
		switch (msg) {
			/*
			 *  Handle activation routines for dealing with activating/
			 *	deactivating windows
			 */
			
			case WM_ACTIVATE:
				if (LOWORD(wp) != WA_INACTIVE) {
					if (v && v->fViewHWND) v->DoActivate(1);
				} else {
					if (v && v->fViewHWND) v->DoActivate(0);
				}
				break;
				
			/*
			 *  Handle button processing
			 */
			
			case WM_LBUTTONDOWN:
			case WM_LBUTTONDBLCLK:
			case WM_RBUTTONDOWN:
			case WM_RBUTTONDBLCLK:
			case WM_MBUTTONDOWN:
			case WM_MBUTTONDBLCLK:
				if (w == GetCapture()) break;
				
				gMods = 0;
				if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
				if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
				if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
				if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
				if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
				if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				
				if ((msg == WM_LBUTTONDBLCLK) || (msg == WM_MBUTTONDBLCLK) ||
						(msg == WM_RBUTTONDBLCLK))
					gMods |= KDoubleClick;
	
				if (v) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					if (v->DoMouseDown(pt,gMods)) {
						SetCapture(w);
					}
				}
				break;
				
			case WM_MOUSEMOVE:
				if (w == GetCapture()) {
					gMods |= KCapturedMouse;
				} else {
					gMods = 0;
					if (GetKeyState(VK_SHIFT) & (~1)) gMods |= KKeyShift;
					if (GetKeyState(VK_CONTROL) & (~1)) gMods |= KKeyControl;
					if (GetKeyState(VK_MENU) & (~1)) gMods |= KKeyMenu;
					if (GetKeyState(VK_LBUTTON) & (~1)) gMods |= KKeyButton;
					if (GetKeyState(VK_MBUTTON) & (~1)) gMods |= KKeyMButton;
					if (GetKeyState(VK_RBUTTON) & (~1)) gMods |= KKeyRButton;
				}
				if (v) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					v->DoMouseMove(pt,gMods);
				}
				break;
				
			case WM_LBUTTONUP:
			case WM_RBUTTONUP:
			case WM_MBUTTONUP:
				if (w == GetCapture()) {
					pt.h = LOWORD(lp);
					pt.v = HIWORD(lp);
					v->DoMouseUp(pt,gMods);
					ReleaseCapture();
				}
				break;
				
			/*
			 *  Handle updating the display
			 */
			
			case WM_PAINT:
				/*
				 *  Draw the contents of this thing
				 */
				
				BeginPaint(w,&ps);
				
				if (v) {
//					newPalette = tw->GetWindowPalette();
//					if (newPalette) {
//						oldPalette = SelectPalette(ps.hdc,newPalette,FALSE);
//						RealizePalette(ps.hdc);
//						v->ProcessUpdateEvent(ps.rcPaint);
//						SelectPalette(ps.hdc,oldPalette,FALSE);
//						DeleteObject(newPalette);
//					} else {
						v->DoDrawView(ps.rcPaint);
//					}
				}
				EndPaint(w,&ps);
				break;
			
			/*
			 *  Scroll bar processing
			 */
			
			case WM_COMMAND:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				}
				break;
				
			case WM_VSCROLL:
			case WM_HSCROLL:
				control = (HWND)lp;
				if (control) {
					XGView *v;
					/* If this is a control message, simply package the */
					/* message and punt it up to the owner view */
					
					v = (XGView *)GetProp(control,_GViewClass);
					if (v) {
						XGSWInternalRecord i;
						
						i.w = w;
						i.msg = msg;
						i.wp = wp;
						i.lp = lp;
						
						ret = v->ReceiveDispatch(KEventWInternal,0,(void *)&i);
					}
				}
				break;
				
			/*
			 *  Default processing
			 */
			
			default:
				defFlag = 1;
				break;
		}
	
		if (defFlag) return DefWindowProc(w,msg,wp,lp);
		else return ret;
	}
	catch (XError &e) {
		e.DisplayError();
	}
	return 0;
}
