/*	XGAppCore_win.cpp
 *
 *		This is the application core for Windows. If this doesn't seem
 *	like much compared to the Mac, that's because it ain't. The actual work
 *	is done in the three callback procs defined elsewhere
 */

#include <ctype.h>
#include <XError.h>
#include <XWindow.h>
#include <XApplication.h>
#include "factory.h"

/************************************************************************/
/*																		*/
/*	Globals																*/
/*																		*/
/************************************************************************/

XGAppCore		*_GApp = NULL;

static XGIViewFactory *GFactory;

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGAppCore::XGAppCore
 *
 *		Create me
 */

XGAppCore::XGAppCore() : XGFocus(this)
{
	/*
	 *	Initialize view factory
	 */
	
	GFactory = new XGIViewFactory;
	
	/*
	 *	Set the pointers up
	 */
	
	if (_GApp) throw XPostError("XGAppCore already created");
	_GApp = this;
	
	/*
	 *	Initialization
	 */
	
	fQuitFlag = false;
	_fModalWindow = NULL;
}

/*	XGAppCore::~XGAppCore
 *
 *		Release me
 */

XGAppCore::~XGAppCore()
{
	if (_GApp == this) {
		_GApp = NULL;
		delete GFactory;
	}
}

/************************************************************************/
/*																		*/
/*	Initialization														*/
/*																		*/
/************************************************************************/

/*	XGAppCore::CommandLine
 *
 *		Do the command line. By default, nothing
 */

void XGAppCore::CommandLine(short, char **)
{
}

/*	XGAppCore::FilterEvent
 *
 *		The opportunity to filter events. By default, nothing
 */

bool XGAppCore::FilterEvent(MSG *)
{
	return false;
}

/************************************************************************/
/*																		*/
/*	Event Loop Processing												*/
/*																		*/
/************************************************************************/

static HACCEL	GAccel;

/*	XGAppCore::Run
 *
 *		Run until it's time to quit
 */

void XGAppCore::Run(void)
{
	MSG msg;
	
	/*
	 *	Parse my command line
	 */
	
	CallCommandLine();
	
	/*
	 *	Initialize for processing events
	 */
	
	GAccel = LoadAccelerators(_GInstance,MAKEINTRESOURCE(128));
	
	/*
	 *	Now run my event loop
	 */
	
	XGBroadcast::SendBroadcast(KEventPostPeriodic,0,NULL);
	while (!fQuitFlag) {
		// Send the idle event
		XGBroadcast::SendBroadcast(KEventPrePeriodic,0,NULL);

		// Get the macintosh event
		if (!GetMessage(&msg,NULL,0,0)) fQuitFlag = true;

		// Get the idle event loop
		_GetCurFocus()->SendDispatch(KEventIdle,0,NULL);
		
		// Translate system accelerator
		if (!_GMDIWindow || !TranslateMDISysAccel(_GMDIWindow,&msg)) {
			// Translate accelerator
			if (!GAccel || !_GMainWindow || !TranslateAccelerator(_GMainWindow,GAccel,&msg)) {
				TranslateMessage(&msg);
				// Filter this event
				if (!FilterEvent(&msg)) {
					DispatchMessage(&msg);
				}
			}
		}
		// Set up focus if it's wrong
		UpdateFocus();
		
		// Send the post idle event
		XGBroadcast::SendBroadcast(KEventPostPeriodic,0,NULL);
	}
}

/*	XGAppCore::_RunOnce
 *
 *		This is an internal routine used for modal window support; this
 *	runs an event loop once with the specified window as the modal window
 */

bool XGAppCore::_RunOnce(XGWindow *w)
{
	MSG msg;
	
	_fModalWindow = w;					// I'm the modal window
	
	// Send the idle event
	XGBroadcast::SendBroadcast(KEventPrePeriodic,0,NULL);

	// Get the macintosh event
	if (!GetMessage(&msg,NULL,0,0)) fQuitFlag = true;

	// Get the idle event loop
	_GetCurFocus()->SendDispatch(KEventIdle,0,NULL);
	
	// Translate system accelerator
	if (!_GMDIWindow || !TranslateMDISysAccel(_GMDIWindow,&msg)) {
		// Translate accelerator
		if (!GAccel || !_GMainWindow || !TranslateAccelerator(_GMainWindow,GAccel,&msg)) {
			TranslateMessage(&msg);
			// Filter this event
			if (!FilterEvent(&msg)) {
				DispatchMessage(&msg);
			}
		}
	}
	
	// Set up focus if it's wrong
	UpdateFocus();
		
	// Send the post idle event
	XGBroadcast::SendBroadcast(KEventPostPeriodic,0,NULL);

	_fModalWindow = NULL;
	return fQuitFlag;
}

/*	XGAppCore::Exit
 *
 *		Exit this application
 */

void XGAppCore::Exit(void)
{
	fQuitFlag = true;
}

/*	XGAppCore::Quit
 *
 *		Handle the quit command
 */

long XGAppCore::Quit(void)
{
	XGWindow *tw;
	
	/*
	 *	Run through and close all of the windows that are open in the
	 *	MDI child list
	 */
	
	for (;;) {
		tw = XGWindow::GetFrontWindow();
		if (tw == NULL) break;				// Done closing all of 'em
		if (!tw->CloseWindow()) break;		// Can't close this thing
	}
	
	if (tw != NULL) return 0;				// Failure to close this thing
	Exit();									// Exit this thing
	return 1;								// And quit.
}

/************************************************************************/
/*																		*/
/*	Event Processing													*/
/*																		*/
/************************************************************************/

/*	XGAppCore::ShowCopyright
 *
 *		Show copyright. ###TODO
 */

void XGAppCore::ShowCopyright()
{
	::MessageBox(_GMainWindow,"this is a blank copyright","hi!",MB_OK);
}

/*	XGAppCore::ReceiveDispatch
 *
 *		handle dispatched commands
 */

long XGAppCore::ReceiveDispatch(long msg, long arg, void *parg)
{
	XGSMenuStatusRecord *gm;
	
	switch (msg) {
		case KEventDoMenuCommand:
			/*
			 *	Handle the following menu commands:
			 *
			 *		Quit/Close/About
			 *		Undo/Cut/Copy/Paste/Clear (desk accessories)
			 */
			
			if (arg == KMenuAboutMe) {
				ShowCopyright();
				return 1;
			}
			if (IsModalWindow()) break;
			if (arg == KMenuFileQuit) {
				Quit();
				return 1;
			}
			
			if (arg == KMenuFileClose) {
				XGWindow::GetFrontWindow()->CloseWindow();
				return 1;
			}
			break;
		
		case KEventGetMenuStatus:
			/*
			 *	Handle enabling those I can handle
			 */
			
			gm = (XGSMenuStatusRecord *)parg;
			
			if (arg == KMenuAboutMe) {
				gm->enable = true;
				return 1;
			}
			if (IsModalWindow()) break;
			if (arg == KMenuFileQuit) {
				gm->enable = true;
				return 1;
			}
			
			if (arg == KMenuFileClose) {
				if (XGWindow::GetFrontWindow()) {
					gm->enable = true;
					return 1;
				}
			}
			break;
	
		default:
			break;
	}
	
	return XGDispatch::ReceiveDispatch(msg,arg,parg);
}


/************************************************************************/
/*																		*/
/*	Parse the command line												*/
/*																		*/
/************************************************************************/

/*	XGAppCore::CallCommandLine
 *
 *		This parses the command line
 */

void XGAppCore::CallCommandLine(void)
{
	char *c;
	char *list[64];
	int i;
	
	/*
	 *	Parse the argc/argv list
	 */
	
	c = _GArgString;
	if (c == NULL) return;
	
	i = 0;
	for (;;) {
		while (isspace(*c)) c++;
		if (*c == 0) break;
		
		if (*c == '"') {
			c++;
			list[i++] = c;
			while ((*c != '"') && (*c != 0)) c++;
			if (*c == 0) break;
			*c++ = 0;
		} else {
			list[i++] = c;
			while (!isspace(*c) && (*c != 0)) c++;
			if (*c == 0) break;
			*c++ = 0;
		}
	}
	
	CommandLine(i,list);
}

/*	XGAppCore::UpdateFocus
 *
 *		This makes sure that my internal focus corrisponds to the current
 *	system focus. This gets called every time I dispatch a message just
 *	to make sure things are set up correctly for me.
 */

void XGAppCore::UpdateFocus()
{
	HWND focus;
	XGView *v;
	XGWindow *w;
	
	focus = ::GetFocus();
	if (focus == NULL) return;
	
	if (NULL != (v = (XGView *)::GetProp(focus,_GViewClass))) {
		/* Yank the library focus here */
		if (!(v->HasFocus())) v->SetFocus();
	} else if (NULL != (w = (XGWindow *)::GetProp(focus,_GWindowClass))) {
		/* Yank Window's focus to the one who has focus */
		if (w->GetFocus() != w) {
			v = dynamic_cast<XGView *>(w->GetFocus());
			if (v) {
				::SetFocus(v->_GetHWND());
			}
		}
	}
}

