/*	XGAppSingleWindow.cpp
 *
 *		This creates a single-window application.
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XWindow.h>
#include <XMenu.h>
#include <XError.h>
#include <XApplication.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGAppSingleWindow::XGAppSingleWindow
 *
 *		Create me
 */

XGAppSingleWindow::XGAppSingleWindow()
{
	fMyWindow = NULL;
	fMyMenuBar = NULL;

#if OPT_WINOS == 1
	_GMainWindow = NULL;
	_GMDIWindow = NULL;				/* Make sure this is clear */
	if (_GPrevInstance == NULL) _InitApplication();
	_InitInstance();				/* Start up instance */
#endif
}

/*	XGAppSingleWindow:::~XGAppSingleWindow
 *
 *		Delete me
 */

XGAppSingleWindow::~XGAppSingleWindow()
{
	if (fMyWindow) delete fMyWindow;
	if (fMyMenuBar) delete fMyMenuBar;
}

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGAppSingleWindow::Run
 *
 *		This verifies that a window was in fact created. If not, this
 *	complains and quits
 */

void XGAppSingleWindow::Run(void)
{
	if (fMyWindow == NULL) throw XPostError("Main window needs to be built");
	XGAppCore::Run();
}

/*	XGAppSingleWindow::ReceiveBroadcast
 *
 *		Process postperiodic event to update menu bar
 */

void XGAppSingleWindow::ReceiveBroadcast(long msg, long, void *)
{
	switch (msg) {
		case KEventPostPeriodic:
			/*
			 *	Update the menu bar
			 */
			
			if ((fMyMenuBar != NULL) && (fMyWindow != NULL)) {
				fMyMenuBar->_LoadFromResource(fMyWindow->GetMenuBarID());
				fMyMenuBar->_UpdateMenuBar(fMyWindow);
			}
			break;
	}
}

/*	XGAppSingleWindow::ReceiveDispatch
 *
 *		This handles receiving various messages, and manages my application
 *	core based on the window construction/destruction messages
 */

long XGAppSingleWindow::ReceiveDispatch(long msg, long arg, void *parg)
{
	XGWindow *w;
	
	switch (msg) {
		case KEventWindowCreate:
			w = (XGWindow *)parg;	
			if (w->GetWindowType() != KWinTypeDocument) break;

			/*
			 *	Record the creation of this window. We only allow one
			 */
			
			if (fMyWindow) throw XPostError("Window already created");
			fMyWindow = w;
			
			/*
			 *	Now that we have a window, initialize the menu bar
			 *	that is associated with this menu
			 */
			
#if OPT_MACOS == 1
			fMyMenuBar = new XGMenuBar();
#endif

#if OPT_WINOS == 1
			_GMainWindow = w->_GetHWND();
			fMyMenuBar = new XGMenuBar(false,_GMainWindow);
#endif
			fMyMenuBar->_LoadFromResource(fMyWindow->GetMenuBarID());
			break;
		
		case KEventWindowDestroy:
			w = (XGWindow *)parg;	
			if (w->GetWindowType() != KWinTypeDocument) break;

			/* 
			 *  Record destruction of this window. Quit if it's mine 
			 */
			
			if (fMyWindow == w) {
				fMyWindow = NULL;

#if OPT_WINOS == 1
				_GMainWindow = NULL;
#endif
				Exit();						// Force exit
			}
			break;
	}
		
	/*
	 *	Pass event upwards
	 */
	
	return XGAppCore::ReceiveDispatch(msg,arg,parg);
}

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGAppSingleWindow::_GetCurMenu
 *
 *		The current menu for this single window application
 */

XGMenuBar *XGAppSingleWindow::_GetCurMenu()
{
	return fMyMenuBar;
}

/*	XGAppSingleWindow::_GetCurFocus
 *
 *		The current menu for this single window application
 */

XGFocus *XGAppSingleWindow::_GetCurFocus()
{
	return fMyWindow;
}

/************************************************************************/
/*																		*/
/*	Windows initialization support										*/
/*																		*/
/************************************************************************/

#if OPT_WINOS == 1

/*	XGAppSingleWinodw::_InitApplication
 *
 *		Register the classes which are in me
 */

void XGAppSingleWindow::_InitApplication(void)
{
	WNDCLASS wc;
	
	/*
	 *	Initialize three instances: a class of class name "_XGWindow"
	 *	and a class of class name "_XGView", and "_XGDialog"
	 */
	
	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = _GSingleWindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = _GInstance;
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = _GWindowClass;
	if (!RegisterClass(&wc)) {
		throw XPostError("Unable to register ^S",_GWindowClass);
	}
	
	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = _GSingleWindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = _GInstance;
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = _GDlogClass;
	if (!RegisterClass(&wc)) {
		throw XPostError("Unable to register ^S",_GDlogClass);
	}
	
	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = _GViewWindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = _GInstance;
	wc.hIcon = NULL;
	wc.hCursor = NULL;
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = _GViewClass;
	if (!RegisterClass(&wc)) {
		throw XPostError("Unable to register ^S",_GViewWindowProc);
	}
}

/*	XGAppSingleWindow::_InitInstance
 *
 *		Initialize the instance of this thing. This does nothing; we assume
 *	the window that is initialized elsewhere will provide me the UI.
 */

void XGAppSingleWindow::_InitInstance(void)
{
}


#endif
