/*	XPostError.cpp
 *
 *		This provides the declaration of the XPostError class
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XError.h>
#include <XConfig.h>
#include <XDataUtil.h>

#include <stdarg.h>

#if OPT_WINOS == 1
#include <windows.h>
#endif

/************************************************************************/
/*																		*/
/*	Internal resources													*/
/*																		*/
/************************************************************************/

#if OPT_MACOS == 1

/*	DITLRes
 *
 *		The point is so I don't have to use any external resources to
 *	display my alert window
 */

static unsigned char DITLRes[] = {
	0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4C, 
	0x01, 0x06, 0x00, 0x60, 0x01, 0x40, 0x04, 0x02, 
	0x4F, 0x4B, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 
	0x00, 0x50, 0x00, 0x3E, 0x01, 0x40, 0x88, 0x02, 
	0x5E, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 
	0x00, 0x18, 0x00, 0x2E, 0x00, 0x38, 0xA0, 0x02, 
	0x00, 0x00
};

#endif

/************************************************************************/
/*																		*/
/*	Conversion Routines													*/
/*																		*/
/************************************************************************/

/*	itos
 *
 *		Convert from long integer to string. This does the conversion.
 */

static char *itos(long l, char *s)
{
	long j;
	
	/*
	 *	+/- flag
	 */
	
	if (l < 0) {
		*s++ = '-';
		l = -l;
	}
	
	/*
	 *	Find base10 number position
	 */
	
	j = 10;
	while (j <= l) j *= 10;
	j /= 10;
	while (j > 0) {
		*s++ = '0' + (l / j);
		l = l % j;
		j /= 10;
	}
	
	/*
	 *	Null terminate and return the new string position
	 */
	
	*s = 0;
	return s;
}

/*	format
 *
 *		This handles a simple formatting of the string
 */

static void format(char *dest, char *arg, va_list vlist)
{
	char *sarg;
	long larg;
	
	while (*arg) {
		if (*arg == '^') {
			arg++;
			if (*arg == 0) break;		/* Skip ^\0 */
			if (*arg == 'S') {
				/* Concatenate string */
				sarg = va_arg(vlist,char *);
				while (*sarg) *dest++ = *sarg++;
			} else if (*arg == 'D') {
				/* Concatenate string */
				larg = va_arg(vlist,long);
				dest = itos(larg,dest);
			} else {
				*dest++ = *arg++;
			}
			arg++;
		} else {
			*dest++ = *arg++;
		}
	}
	*dest = 0;
}

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XPostError::XPostError
 *
 *		Construct this
 */

XPostError::XPostError(char *msg,...)
{
	va_list vlist;
	
	va_start(vlist,msg);
	format(fBuffer,msg,vlist);
	va_end(vlist);
}

/*	XPostError::~XPostError
 *
 *		Destruct this
 */

XPostError::~XPostError()
{
}

/************************************************************************/
/*																		*/
/*	Error presentation routine											*/
/*																		*/
/************************************************************************/

/*	XPostError::DisplayError
 *
 *		Display this error. This displays the message on the display,
 *	using no resources. (Which is a trick on the Macintosh...)
 */

void XPostError::DisplayError()
{
#if OPT_WINOS == 1
	MessageBox(NULL,fBuffer,"Error",MB_OK | MB_ICONSTOP);
#endif

#if OPT_MACOS == 1
	Handle h;
	Handle tmph;
	Rect r;
	DialogRecord d;
	DialogPtr dptr;
	unsigned char buffer[80];
	short i;
	
	/*
	 *	Get the DITL resource into a handle
	 */
	
	h = NewHandle(sizeof(DITLRes));
	BlockMove(DITLRes,*h,sizeof(DITLRes));
	
	/*
	 *	Set up the paramtext parameter
	 */
	
	ParamText(cvtc2p(buffer,fBuffer),NULL,NULL,NULL);
	
	/*
	 *	Put me in the upper left of the screen
	 */
	
	r.left = 112;
	r.top = 76;
	r.right = r.left + 334;	/* Width */
	r.bottom = r.top + 110; /* Height */
	
	dptr = NewDialog((Ptr)&d,&r,"\p",true,1,(WindowPtr)-1L,false,0L,h);
	
	/*
	 *	Get the default item and circle it
	 */
	
	SetPort(dptr);
	GetDialogItem(dptr,1,&i,&tmph,&r);
	PenSize(3,3);
	InsetRect(&r,-4,-4);
	FrameRoundRect(&r,16,16);
	
	/*
	 *	Run the event loop
	 */
	
	for (;;) {
		ModalDialog(NULL,&i);
		if (i == 1) break;
	}
	
	/*
	 *	Close the dialog box
	 */
	
	CloseDialog(dptr);
	DisposeHandle(h);
#endif
}
