/*	XGFocus.cpp
 *
 *		This is the code to handle managing multiple focuses. The reason
 *	for this is simple: each window (and the application itself) can have
 *	a focus--thus, this allows each to have it's own focus.
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XEvent.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGFocus::XGFocus
 *
 *		Make me
 */

XGFocus::XGFocus(XGDispatch *x)
{
	fDefault = x;
	_SetFocus(x);
	fDefault->_SetFocusDispatch(this);
}

/*	XGFocus::~XGFocus
 *
 *		Kill me
 */

XGFocus::~XGFocus()
{
}

/************************************************************************/
/*																		*/
/*	Focus Manipulation													*/
/*																		*/
/************************************************************************/

/*	XGFocus::_SetFocus
 *
 *		This sets the focus of this thing.
 */

void XGFocus::_SetFocus(XGDispatch *x)
{
	fFocus = x;
}

/*	XGFocus::_ClearFocus
 *
 *		When the current focus dies, this knows how to move the focus
 *	to the default object
 */

void XGFocus::_ClearFocus()
{
	if (fFocus == fDefault) {
		fDefault = NULL;					// I'm dying! I'm dying!
		fFocus = NULL;
	} else {
		_SetFocus(fFocus);
		fFocus->GainFocus();
	}
}

/************************************************************************/
/*																		*/
/*	Message Dispatch													*/
/*																		*/
/************************************************************************/

/*	XGFocus::SendDispatch
 *
 *		Send this message
 */

long XGFocus::SendDispatch(long m, long arg, void *parg)
{
	if (fFocus) return fFocus->ReceiveDispatch(m,arg,parg);
	else return 0;
}

