/*	XGSend.cpp
 *
 *		This is the sender pair of the send/receive classes
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XEvent.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGSend::XGSend
 *
 *		Construct me.
 */

XGSend::XGSend()
{
}

/*	XGSend::~XGSend
 *
 *		Destroy me. This walks through and removes all of the receivers
 *	that are attached to me
 */

XGSend::~XGSend()
{
	while (fReceive.Length()) {
		RemoveReceiver(fReceive[0]);
	}
}

/************************************************************************/
/*																		*/
/*	Receiver management													*/
/*																		*/
/************************************************************************/

/*	XGSend::AddReceiver
 *
 *		Add this to my list of receivers
 */

void XGSend::AddReceiver(XGReceive *r)
{
	long l,o;
	
	l = fReceive.Length();
	for (o = 0; o < l; o++) {
		if (fReceive[o] == r) return;	// Already inserted in this
	}
	
	fReceive += r;
	r->fSend += this;
}

/*	XGSend::RemoveReciever
 *
 *		Remove this receiver from this list. This also removes the
 *	backpointer
 */

void XGSend::RemoveReceiver(XGReceive *r)
{
	long l,o;
	
	l = fReceive.Length();
	for (o = 0; o < l; o++) {
		if (fReceive[o] == r) {
			fReceive.Delete(o);
			break;
		}
	}
	
	l = (r->fSend).Length();
	for (o = 0; o < l; o++) {
		if ((r->fSend)[o] == this) {
			(r->fSend).Delete(o);
			break;
		}
	}
}

/************************************************************************/
/*																		*/
/*	Message Dispatch													*/
/*																		*/
/************************************************************************/

/*	XGSend::SendMessage
 *
 *		Send this message to all the receivers
 */

void XGSend::SendMessage(long msg, long arg, void *parg)
{
	long l,o;
	
	l = fReceive.Length();
	for (o = 0; o < l; o++) {
		fReceive[o]->ReceiveMessage(msg,arg,parg);
	}
}

