/*	XGMenuBar.cpp
 *
 *		This is the menu bar engine itself; this encapsulates the manipulation
 *	of the menu bar's contents itself
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XConfig.h>
#include <XThread.h>
#include <XMenu.h>
#include <XError.h>
#include <XEvent.h>
#include <XApplication.h>
#include <XDataUtil.h>

/************************************************************************/
/*																		*/
/*	Construction/destruction											*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::XGMenuBar
 *
 *		Don't do much
 */

#if OPT_MACOS == 1
XGMenuBar::XGMenuBar()
{
	fCurResID = -1;
}
#endif

#if OPT_WINOS == 1
XGMenuBar::XGMenuBar(bool f, HWND w)
{
	fMDIWindowFlag = f;
	fMenuWindow = w;
	fMenuHandle = NULL;
	fCurResID = -1;
}
#endif

/*	XGMenuBar::~XGMenuBar
 *
 *		Fry this thing in my own creative way
 */

XGMenuBar::~XGMenuBar()
{
#if OPT_MACOS == 1
	int len,off;
	
	len = fCommands.Length();
	for (off = 0; off < len; off++) delete fCommands[off];
#endif
}
	
/************************************************************************/
/*																		*/
/*	Resource manipulation												*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::_LoadFromResource
 *
 *		This is the routine to load from the resource. The resources
 *	that are used for loading this thing are:
 *
 *		'MBAR' #XXX		The menu bar resource
 *		'MInv' #XXX		The popup items in the menu bar
 *		'MENU' #YYY		The menu specification
 *		'MCmd' #YYY		The menu command codes for this menu
 */

void XGMenuBar::_LoadFromResource(short id)
{
	/*
	 *	The Macintosh version of this code loads the resources and
	 *	all the auxiliary stuff that gives my menus menu command IDs
	 */
	
#if OPT_MACOS == 1
	long len,off;
	short **mbar,**minv;
	MenuHandle mh;
	_MenuIDArray *cmd;
	long **mcmd;
	long imenu,lmenu;
	Handle h;
	
	/*
	 *	Fast sanity check
	 */
	
	if (id == fCurResID) return;
	fCurResID = id;
	
	/*
	 *	Delete the prior contents and release the memory
	 */
	
	::ClearMenuBar();
	len = fMenuBar.Length();
	for (off = 0; off < len; off++) ::ReleaseResource((Handle)fMenuBar[off]);
	len = fHidden.Length();
	for (off = 0; off < len; off++) ::ReleaseResource((Handle)fHidden[off]);
	len = fCommands.Length();
	for (off = 0; off < len; off++) delete fCommands[off];
	
	/*
	 *	Now start loading the resources--first, the invisible ones
	 */
	
	minv = (short **)::GetResource('MInv',id);
	if (minv != NULL) {
		/*
		 *	If there are any invisible menus, load them
		 */
		
		::HLock((Handle)minv);
		for (off = 1; off <= **minv; off++) {
			// Load the menu and insert it into my list of menus
			mh = ::GetMenu((*minv)[off]);
			fHidden += mh;
			::InsertMenu(mh,-1);			// Invisible popup
			
			// Load the command list with this menu
			mcmd = (long **)::GetResource('MCmd',(*minv)[off]);
			if (mcmd == NULL) throw XPostError("Resource 'MCmd'#^D missing",(long)((*minv)[off]));
			
			// Copy this thing over
			::HLock((Handle)mcmd);
			cmd = new _MenuIDArray;
			cmd->menuID = (*minv)[off];
			lmenu = ::CountMItems(mh);
			for (imenu = 0; imenu < lmenu; imenu++) {
				(cmd->itemID) += (*mcmd)[imenu];
			}
			::HUnlock((Handle)mcmd);
			::ReleaseResource((Handle)mcmd);
			
			// Load this into my translation table
			fCommands += cmd;
		}
		::HUnlock((Handle)minv);
		::ReleaseResource((Handle)minv);
	}
	
	/*
	 *	And now load the visible ones
	 */
	
	mbar = (short **)::GetResource('MBAR',id);	// Parse this myself
	if (mbar == NULL) throw XPostError("Menu resource 'MBAR'#^D missing",(long)id);
	::DetachResource((Handle)mbar);
	
	h = GetNewMBar(id);
	SetMenuBar(h);
	
	/*
	 *	Load the visible commands
	 */
	
	::HLock((Handle)mbar);
	for (off = 1; off <= **mbar; off++) {
		// Load the menu and insert it into my list of menus
		mh = ::GetMenu((*mbar)[off]);
		fMenuBar += mh;
		// <Already inserted by SetMenuBar() routine>
		
		// Load the command list with this menu
		mcmd = (long **)::GetResource('MCmd',(*mbar)[off]);
		if (mcmd == NULL) throw XPostError("Resource 'MCmd'#^D missing",(long)((*mbar)[off]));
		
		// Copy this thing over
		::HLock((Handle)mcmd);
		cmd = new _MenuIDArray;
		cmd->menuID = (*mbar)[off];
		lmenu = ::CountMItems(mh);
		for (imenu = 0; imenu < lmenu; imenu++) {
			(cmd->itemID) += (*mcmd)[imenu];
		}
		::HUnlock((Handle)mcmd);
		::ReleaseResource((Handle)mcmd);
		
		// Load this into my translation table
		fCommands += cmd;
	}
	/*
	 *	Install the desk accessories
	 */
	
	fApplMenu = (*mbar)[1];						// remember application menu
	mh = ::GetMenu(fApplMenu);
	fApplLen = CountMItems(mh);					// and remember it's length	
	AppendResMenu(mh,'DRVR');

	/*
	 *	Now, draw the menubar
	 */
	
	::HUnlock((Handle)mbar);
	::DisposeHandle((Handle)mbar);
	
	::DrawMenuBar();
#endif

	/*
	 *	The WinOS version is a hell of a lot easier--we use the 
	 *	built-in menu handling stuff
	 */
	
#if OPT_WINOS == 1
	HMENU old;
	
	if (id == fCurResID) return;
	
	old = fMenuHandle;
	fCurResID = id;
	fMenuHandle = ::LoadMenu(_GInstance,MAKEINTRESOURCE(id));
	
	if (fMDIWindowFlag) {
		SendMessage(fMenuWindow,WM_MDISETMENU,(WPARAM)fMenuHandle,NULL);
	} else {
		::SetMenu(fMenuWindow,fMenuHandle);
	}
	if (old) ::DestroyMenu(old);
	
	if (fMDIWindowFlag) {
		SendMessage(fMenuWindow,WM_MDIREFRESHMENU,0,0);
	}
	::DrawMenuBar(fMenuWindow);
#endif
}

/************************************************************************/
/*																		*/
/*	Menu Manipulation													*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::GetNumMenus
 *
 *		Get the number of menus in this particular submenu
 */

long XGMenuBar::GetNumMenus(void)
{
#if OPT_MACOS == 1
	return fMenuBar.Length();
#endif

#if OPT_WINOS == 1
	/*
	 *	The windows version needs to knock off two if this thing is
	 *	zoomed. Which means we need to know if we are zoomed or not
	 */
	
	long count;
	
	count = ::GetMenuItemCount(fMenuHandle);
	if (IsZoomed()) count -= 2;			/* Close and zoom menus */
	return count;
#endif
}

/*	XGMenuBar::GetMenu
 *
 *		Get this specific menu
 */

long XGMenuBar::GetMenu(short id)
{
#if OPT_MACOS == 1
	return (long)(fMenuBar[id]);
#endif

#if OPT_WINOS == 1
	if (IsZoomed()) id--;
	return (long)::GetSubMenu(fMenuHandle,id);
#endif
}

/************************************************************************/
/*																		*/
/*	Menu content manipulation											*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::GetNumItems
 *
 *		Get the number of items in this thing
 */

short XGMenuBar::GetNumItems(long m)
{
#if OPT_MACOS == 1
	return ::CountMItems((MenuHandle)m);
#endif

#if OPT_WINOS == 1
	return ::GetMenuItemCount((HMENU)m);
#endif
}

/*	XGMenuBar::GetItemID
 *
 *		Get the ID of this menu handle
 */

long XGMenuBar::GetItemID(long m, short index)
{
#if OPT_MACOS == 1
	_MenuIDArray *ma;
	
	// Look up the command array based on the menu's internal ID
	ma = GetMenuArray((**((MenuHandle)m)).menuID);
	
	// Now return the command
	if ((index < 0) || (index >= ma->itemID.Length())) return -1;
	return ma->itemID[index];
#endif

#if OPT_WINOS == 1
	return ::GetMenuItemID((HMENU)m,index);
#endif
}

/*	XGMenuBar::SetMenuID
 *
 *		Set the ID for this thing
 */

void XGMenuBar::SetItemID(long m, short index, long cmd)
{
#if OPT_MACOS == 1
	_MenuIDArray *ma;
	
	// Look up the command array based on the menu's internal ID
	ma = GetMenuArray((**((MenuHandle)m)).menuID);
	
	// Now return the command
	if ((index < 0) || (index >= ma->itemID.Length())) return;
	ma->itemID[index] = cmd;
#endif

#if OPT_WINOS == 1
	MENUITEMINFO i;
	
	i.cbSize = sizeof(i);
	i.fMask = MIIM_ID;
	i.wID = cmd;
	::SetMenuItemInfo((HMENU)m,index,TRUE,&i);
#endif
}

/*	XGMenuBar::GetItemText
 *
 *		Get the item text in this menu
 */

void XGMenuBar::GetItemText(long m, short index, char *array)
{
#if OPT_MACOS == 1
	unsigned char buffer[256];
	
	::GetMenuItemText((MenuHandle)m,index+1,buffer);
	cvtp2c(array,buffer);
#endif

#if OPT_WINOS == 1
	MENUITEMINFO i;
	
	i.cbSize = sizeof(i);
	i.fMask = MIIM_TYPE;
	i.fType = MFT_STRING;
	i.cch = 255;
	i.dwTypeData = (LPTSTR)array;
	::GetMenuItemInfo((HMENU)m,index,TRUE,&i);
#endif
}

/*	XGMenuBar::SetItemText
 *
 *		Set the item text in this thing
 */

void XGMenuBar::SetItemText(long m, short index, char *array)
{
#if OPT_MACOS == 1
	unsigned char buffer[256];
	
	if (*array == 0) {
		/* Work around Menu Manager bug: zero-length text truncates menu */
		::SetMenuItemText((MenuHandle)m,index+1,"\p ");
	} else {
		::SetMenuItemText((MenuHandle)m,index+1,cvtc2p(buffer,array));
	}
#endif

#if OPT_WINOS == 1
	MENUITEMINFO i;
	
	i.cbSize = sizeof(i);
	i.fMask = MIIM_TYPE;
	i.fType = MFT_STRING;
	i.cch = 255;
	i.dwTypeData = (LPTSTR)array;
	::SetMenuItemInfo((HMENU)m,index,TRUE,&i);
#endif
}

/*	XGMenuBar::GetItemShortcut
 *
 *		Get the item's shortcut
 */

char XGMenuBar::GetItemShortcut(long m, short index)
{
#if OPT_MACOS == 1
	short c;
	
	::GetItemCmd((MenuHandle)m,index+1,&c);
	if (c <= 0x20) c = 0;
	return c;
#endif

#if OPT_WINOS == 1
	#pragma unused(m,index)
//###TODO: Modify the accelerator shortcut
	return 0;
#endif
}

/*	XGMenuBar::SetItemShortcut
 *
 *		Set this thing's shortcut
 */

void XGMenuBar::SetItemShortcut(long m, short index, char c)
{
#if OPT_MACOS == 1
	if (c <= 0x20) throw XPostError("Illegal shortcut command");
	::SetItemCmd((MenuHandle)m,index+1,c);
#endif

#if OPT_WINOS == 1
	#pragma unused(m,index,c)
//###TODO: Modify the accelerator shortcut
#endif
}

/*	XGMenuBar::GetSubMenu
 *
 *		Get the submenu reference from this item, if one exists.
 *	If there isn't one, this returns 0
 */

long XGMenuBar::GetSubMenu(long m, short index)
{
#if OPT_MACOS == 1
	short mid;
	short c;
	
	::GetItemCmd((MenuHandle)m,index+1,&c);
	if (c != 0x1B) return 0;
	::GetItemMark((MenuHandle)m,index+1,&mid);
	if (mid == 0) return 0;
	
	return (long)::GetMenu(mid);			/* Get from resource manager */
#endif

#if OPT_WINOS == 1
	MENUITEMINFO i;
	
	i.cbSize = sizeof(i);
	i.fMask = MIIM_SUBMENU;
	::GetMenuItemInfo((HMENU)m,index,TRUE,&i);
	return (long)(i.hSubMenu);
#endif
}

/************************************************************************/
/*																		*/
/*	Insertion/Deletion													*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::InsertMenuItem
 *
 *		This inserts this menu bar item into the specified menu.
 *	The inserted menu item is initially empty, with a command code
 *	of zero; after performing this operation, you need to set the
 *	text, command, and shortcut yourself.
 *
 *		The index specifies the (0-numbered) location where the
 *	new entry goes; the old entry at that location is bumped up in
 *	it's index location in the menu.
 */

void XGMenuBar::InsertMenuItem(long menu, short index)
{
#if OPT_MACOS == 1
	_MenuIDArray *id;
	long cmd;
	
	/*
	 *	Get the command array
	 */
	
	id = GetMenuArray((**((MenuHandle)menu)).menuID);
	
	/*
	 *	Insert the menu item
	 */
	
	::InsertMenuItem((MenuHandle)menu,"\p(-",index);
	cmd = -1;
	id->itemID.Insert(index,cmd);
#endif

#if OPT_WINOS == 1
	short len;
	
	len = ::GetMenuItemCount((HMENU)menu);
	if (index >= len) index = -1;
	::InsertMenu((HMENU)menu,index,MF_BYPOSITION | MF_SEPARATOR,0,NULL);
#endif
}

/*	XGMenuBar::DeleteMenuItem
 *
 *		Delete this menu bar item. I don't flush the submenu item
 *	in the Macintosh version; I don't care. And I don't leak memory...
 */

void XGMenuBar::DeleteMenuItem(long menu, short index)
{
#if OPT_MACOS == 1
	_MenuIDArray *id;
	
	/*
	 *	Get the command array
	 */
	
	id = GetMenuArray((**((MenuHandle)menu)).menuID);
	
	/*
	 *	Delete the menu item
	 */
	
	::DeleteMenuItem((MenuHandle)menu,index+1);
	id->itemID.Delete(index);
#endif

#if OPT_WINOS == 1
	::RemoveMenu((HMENU)menu,index,MF_BYPOSITION);
#endif
}

/************************************************************************/
/*																		*/
/*	Menu Support														*/
/*																		*/
/************************************************************************/

/*	XGMenuBar::_UpdateMenuBar
 *
 *		This sends to the current focus the commands needed to figure
 *	out if the various items here should be enabled or disabled
 */

void XGMenuBar::_UpdateMenuBar(XGFocus *f)
{
#if OPT_MACOS == 1
	long len,off;
	
	/*
	 *	Run through all the menu bars and call my UpdateMenu engine
	 */
	
	len = fMenuBar.Length();
	for (off = 0; off < len; off++) UpdateMenu(f,fMenuBar[off]);
	len = fHidden.Length();
	for (off = 0; off < len; off++) UpdateMenu(f,fHidden[off]);
#endif

#if OPT_WINOS == 1
    UpdatePopUpMenu(f,fMenuHandle,IsZoomed()); 	// Update the contents
#endif
}

#if OPT_MACOS == 1

/*	XGMenuBar::_DispatchCommand
 *
 *		This is the code to figure out what the command is, and
 *	sends it to the focus element
 */

void XGMenuBar::_DispatchCommand(XGFocus *f, short hi, short lo)
{
	_MenuIDArray *id;
	
	/*
	 *	Is this a DA?
	 */
	
	if ((hi == fApplMenu) && (lo > fApplLen)) {
		unsigned char buffer[256];
		
		::GetMenuItemText(::GetMenu(hi),lo,buffer);
		::OpenDeskAcc(buffer);
		return;
	}
	
	/*
	 *	Dispatch this non-Apple DA command
	 */
	
	id = GetMenuArray(hi);
	f->SendDispatch(KEventDoMenuCommand,id->itemID[lo-1],NULL);
}

#endif // OPT_MACOS


/************************************************************************/
/*																		*/
/*	Internal Support													*/
/*																		*/
/************************************************************************/

#if OPT_MACOS == 1

/*	XGMenuBar::GetMenuArray
 *
 *		Get the menu array associated with this menu identifier
 */

_MenuIDArray *XGMenuBar::GetMenuArray(short id)
{
	long len,off;
	_MenuIDArray *ma;
	
	len = fCommands.Length();
	for (off = 0; off < len; off++) {
		ma = fCommands[off];
		if (ma->menuID == id) return ma;
	}
	throw XPostError("Unknown menu identifier?");
	return NULL;
}

/*	XGMenuBar::UpdateMenu
 *
 *		This is the core of the UpdateMenuBar command; this runs through
 *	and figures out what items in this menu should be hilited and checked
 */

void XGMenuBar::UpdateMenu(XGFocus *f, MenuHandle mh)
{
	_MenuIDArray *ma;
	long len;
	long off;
	XGSMenuStatusRecord msr;
	short c;
	long cmd;
	
	ma = GetMenuArray((**mh).menuID);
	len = ma->itemID.Length();
	for (off = 0; off < len; off++) {
		/*
		 *	Okay, what do I do?
		 */
		
		::GetItemCmd(mh,off+1,&c);
		if (c == 0x1B) {
			// This is a submenu. Enable this always.
			::EnableItem(mh,off+1);
			::SetItemMark(mh,off+1,0);
		} else {
			cmd = ma->itemID[off];
			if (cmd == 0) {
				// This is a separator. Disable
				::DisableItem(mh,off+1);
				::SetItemMark(mh,off+1,0);
			} else {
				// This is a command. Figure out what to do
				msr.enable = false;
				msr.checked = false;
				msr.checkchar = 0;			/* Defaults */
	
				f->SendDispatch(KEventGetMenuStatus,cmd,(void *)&msr);
				
				// Now, based on the results, check and enable stuff
				if (msr.enable) {
					::EnableItem(mh,off+1);
				} else {
					::DisableItem(mh,off+1);
				}
				if (msr.checked) {
					if (msr.checkchar == 0) {
						::SetItemMark(mh,off+1,0x12);	/* Default check */
					} else {
						::SetItemMark(mh,off+1,msr.checkchar);
					}
				} else {
					::SetItemMark(mh,off+1,0);
				}
			}
		}
	}
}

#endif

#if OPT_WINOS == 1

/*	XGMenuBar::IsZoomed
 *
 *		Determine if my host window is zoomed
 */

bool XGMenuBar::IsZoomed()
{
	HWND w;
	
	if (fMDIWindowFlag) {
		w = GetWindow(fMenuWindow,GW_CHILD);
		while (w != NULL) {
			if (GetProp(w,_GWindowClass)) {
				if (IsWindowVisible(w)) break;
			}
			w = GetWindow(w,GW_HWNDNEXT);
		}
		if (w) {
			return ::IsZoomed(w);
		}
	}
	return false;
}

/*	XGMenuBar::UpdatePopUpMenu
 *
 *		This updates the particular pop-up menu
 */
 
int XGMenuBar::UpdatePopUpMenu(XGFocus *f, HMENU popUp, short flag)
{
    short len,item;
    UINT state;
    HMENU subPopUp;
    int anyEnable = 0;
    int anyChange = 0;
    UINT id;
    int temp;
    XGSMenuStatusRecord msr;

    if (flag) flag = 1;			/* True: offset things by 1 */

    len = GetMenuItemCount(popUp) - flag;
    for (item = flag; item < len; item++) {
		subPopUp = ::GetSubMenu(popUp,item);
		if (subPopUp == NULL) {
	
		    // This is not a popup menu.
	
		    id = GetMenuItemID(popUp,item);
		    if ((id == 0xFFFF) || (id == 0)) {
	
				// This is a separator or something... Ignore it
	
				continue;
		    }
	
			if ((id & 0xF000) == 0xF000) {
				// Built-in Win32 Command always enabled and unchecked
				msr.checked = false;
				msr.enable = true;
			} else {
				msr.enable = false;
				msr.checked = false;
				msr.checkchar = 0;
			
				f->SendDispatch(KEventGetMenuStatus,id,(void *)&msr);
			}
			
			temp = msr.enable;

			// Update check state for this particular item.
			if (msr.checked) {
		    	CheckMenuItem(popUp,item,MF_BYPOSITION | MF_CHECKED);
			} else {
		    	CheckMenuItem(popUp,item,MF_BYPOSITION | MF_UNCHECKED);
			}
		} else {
		    // This is a popup menu.
	
#ifdef DISABLEMENUS
		    temp = UpdatePopUpMenu(f,subPopUp,0);	// Disable popup menu if
		    temp = (temp & 0x02) != 0;			// all children disabled
#else
		   	UpdatePopUpMenu(f,subPopUp,0);
		    temp = 1;
#endif
		}
		if (temp) anyEnable = 1;		// Update anyEnable flag
		state = GetMenuState(popUp,item,MF_BYPOSITION);
		if (!(state & (MF_DISABLED | MF_GRAYED))) {
		    if (!temp) {
				anyChange = 1;		// Something changed...
				EnableMenuItem(popUp,item,MF_BYPOSITION | MF_GRAYED);
		    }
		} else {
		    if (temp) {
				anyChange = 1;
				EnableMenuItem(popUp,item,MF_BYPOSITION | MF_ENABLED);
		    }
		}
	}

    return (anyChange ? 0x01 : 0x00) | (anyEnable ? 0x02 : 0x00);
}

#endif
