/*	XGSemaphore.cpp
 *
 *		Semaphore support. This encapsulates semaphore functionality on
 *	the destination architecture
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XThread.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGSemaphore::XGSemaphore
 *
 *		Create this semaphore
 */

XGSemaphore::XGSemaphore(unsigned long max)
{
#if OPT_MACOS == 1
	fAccess = max;
#endif

#if OPT_WINOS == 1
	fHandle = CreateSemaphore(NULL,max,max,NULL);
#endif
}

/*	XGSemaphore::~XGSemaphore
 *
 *		Delete this thing
 */

XGSemaphore::~XGSemaphore()
{
#if OPT_WINOS == 1
	if (fHandle) CloseHandle(fHandle);
#endif
}

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGSemaphore::EnterCritical
 *
 *		Enter critical section that is protected by this semaphore
 */
 
void XGSemaphore::EnterCritical(void)
{
#if OPT_MACOS == 1
	ThreadBeginCritical();				// Stop tasks swap temporarly
	
	while (fAccess == 0) {
		ThreadEndCritical();
		YieldToAnyThread();				// Unable to enter yet.
		ThreadBeginCritical();
	}
	fAccess--;							// I'm in. Allow access again.
	
	ThreadEndCritical();
#endif

#if OPT_WINOS == 1
	WaitForSingleObject(fHandle,INFINITE);
#endif
}

/*	XGSemaphore::LeaveCritical
 *
 *		Leave critical section
 */

void XGSemaphore::LeaveCritical(void)
{
#if OPT_MACOS == 1
	ThreadBeginCritical();				// Stop tasks swap temporarly
	fAccess++;							// Atomic increment allowed access
	ThreadEndCritical();				// Restart task swapping
#endif

#if OPT_WINOS == 1
	ReleaseSemaphore(fHandle,1,NULL);
#endif
}
