/*	CursorUtil.c
 *
 *		Custom cursor utilities
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XCursorUtil.h>
#include <XConfig.h>


/************************************************************************/
/*																		*/
/*	Globals																*/
/*																		*/
/************************************************************************/

static short			*GCursor;
static short			GCurCursor = 0;
static short			GCurWatch = -1;

/************************************************************************/
/*																		*/
/*	Initialization														*/
/*																		*/
/************************************************************************/

/*	InitCursorAnimation
 *
 *		Initialize the cursor animation
 */

void InitCursorAnimation(short *l)
{
	GCursor = l;
}

/************************************************************************/
/*																		*/
/*	Set the cursor stuff												*/
/*																		*/
/************************************************************************/

/*	_SetCursorID
 *
 *		Set the cursor by the specified identifier
 */

static void _SetCursorID(short id)
{
#if OPT_MACOS == 1
	CursHandle c;
	CCrsrHandle cc;
	
	/*
	 *	Macintosh world--first, find the color cursor
	 */
	
	if (id == KNormalCursor) {
		// Trap out this one--KNormalCursor is in global memory
		SetCursor(&qd.arrow);
	} else {
		// Get the color cursor if possible
		cc = GetCCursor(id);
		if (cc) {
			SetCCursor(cc);
			DisposeCCursor(cc);
		} else {
			// Color cursor is not there--try black and white
			c = GetCursor(id);
			if (c) {
				HLock((Handle)c);
				SetCursor(*c);
				HUnlock((Handle)c);
				ReleaseResource((Handle)c);
			} else {
				SetCursor(&qd.arrow);
			}
		}
	}
#endif

#if OPT_WINOS == 1
	HCURSOR cursor;
	HCURSOR oldCursor;
	
	if (id == 0) {					// Arrow cursor
		cursor = LoadCursor(NULL,IDC_ARROW);
	} else if (id == 1) {			// iBeamCursor
		cursor = LoadCursor(NULL,IDC_IBEAM);
	} else if (id == 2) {			// crossCursor
		cursor = LoadCursor(NULL,IDC_CROSS);
	} else if (id == 3) {			// plusCursor
		cursor = LoadCursor(NULL,IDC_CROSS);
	} else if (id == 4) {			// watchCursor
		cursor = LoadCursor(NULL,IDC_WAIT);
	} else {
		cursor = LoadCursor(_GInstance,MAKEINTRESOURCE(id));
		if (cursor == NULL) {
			id = 0;
			cursor = LoadCursor(NULL,IDC_ARROW);
		}
	}
	
	oldCursor = SetCursor(cursor);
	
	if (oldCursor != cursor) {
		DestroyCursor(oldCursor);
	}
#endif
}

/*	SetCursorID
 *
 *		Like the above, but also sets the current cursor (for the wait
 *	cursor stuff)
 */

void SetCursorID(short id)
{
	GCurWatch = -1;
	GCurCursor = id;
	_SetCursorID(id);
}

/*	NormalCursor
 *
 *		Restore the prior cursor after a wait cursor loop
 */

void NormalCursor(void)
{
	if (GCurWatch != -1) {
		GCurWatch = -1;
		_SetCursorID(GCurCursor);
	}
}

/*	WaitCursor
 *
 *		Wait cursor--this spins the cursor animation sequence if any
 */

void WaitCursor(void)
{
	if (GCursor == NULL) {
		/*
		 *	No animation
		 */
		
		if (GCurWatch == -1) {
			GCurWatch = 0;
			_SetCursorID(KWatchCursor);		/* Default without animation */
		}
	} else {
		GCurWatch++;
		if (GCursor[GCurWatch] == 0) GCurWatch = 0;
		_SetCursorID(GCursor[GCurWatch]);
	}
}

