/*	DataUtil.c
 *
 *		Data manipulation utilities
 */
 
/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XDataUtil.h>

/************************************************************************/
/*																		*/
/*	String Conversion													*/
/*																		*/
/************************************************************************/

/*	cvtc2p
 *
 *		Convert from C to Pascal
 */

unsigned char *cvtc2p(unsigned char *dest, const char *src)
{
	register unsigned char *ptr;
	register unsigned char len = 0;
	
	ptr = dest+1;
	while (*src) {
		*ptr++ = (unsigned char)*src++;
		len++;
	}
	*dest = len;
	return dest;
}

/*	cvtp2c
 *
 *		Convert from Pascal to C format string
 */

char *cvtp2c(char *dest, const unsigned char *src)
{
	register char *ptr;
	register unsigned char len;
	
	ptr = dest;
	len = *src++;
	while (len--) *ptr++ = (char)*src++;
	*ptr = 0;
	
	return dest;
}

/************************************************************************/
/*																		*/
/*	Pascal string manipulation											*/
/*																		*/
/************************************************************************/

/*	pstrcpy
 *
 *		string copy operation
 */

unsigned char *pstrcpy(unsigned char *dest, const unsigned char *src)
{
	register unsigned char len;
	register unsigned char *ptr;
	
	len = *src;
	ptr = dest;
	*ptr++ = *src++;
	while (len--) *ptr++ = *src++;
	
	return dest;
}

/*	pstrcat
 *
 *		concatenate pascal strings
 */

unsigned char *pstrcat(unsigned char *dest, const unsigned char *src)
{
	register unsigned char len;
	register unsigned char *ptr;
	
	len = *src++;
	ptr = dest + *dest + 1;
	*dest += len;
	while (len--) *ptr++ = *src++;
	
	return dest;
}

/*	pstrcmp
 *
 *		Compare. This works like the C string compare
 */

long pstrcmp(const unsigned char *a, const unsigned char *b)
{
	register unsigned char alen;
	register unsigned char blen;
	
	alen = *a++;
	blen = *b++;
	while (alen && blen) {
		if (*a < *b) return -1;
		if (*a > *b) return 1;
		alen--;
		blen--;
	}
	if (alen < blen) return 1;
	if (alen > blen) return -1;
	return 0;
}

/************************************************************************/
/*																		*/
/*	Byte Swap Utilities													*/
/*																		*/
/************************************************************************/

/*	SwapWord
 *
 *		Swap this thing
 */

unsigned short SwapWord(unsigned short c)
{
	unsigned char tmp;
	unsigned char *ptr = (unsigned char *)&c;
	
	tmp = ptr[0];
	ptr[0] = ptr[1];
	ptr[1] = tmp;
	
	return c;
}

/*	SwapLong
 *
 *		Swap this thing
 */

unsigned long SwapLong(unsigned long c)
{
	unsigned char tmp;
	unsigned char *ptr = (unsigned char *)&c;
	
	tmp = ptr[0];
	ptr[0] = ptr[3];
	ptr[3] = tmp;
	tmp = ptr[1];
	ptr[1] = ptr[2];
	ptr[2] = tmp;
	
	return c;
}

/*	SwapFloat
 *
 *		Swap this thing
 */

float SwapFloat(float c)
{
	unsigned char tmp;
	unsigned char *ptr = (unsigned char *)&c;
	
	tmp = ptr[0];
	ptr[0] = ptr[3];
	ptr[3] = tmp;
	tmp = ptr[1];
	ptr[1] = ptr[2];
	ptr[2] = tmp;
	
	return c;
}

/*	SwapDouble
 *
 *		Swap this thing
 */

double SwapDouble(double c)
{
	unsigned char tmp;
	unsigned char *ptr = (unsigned char *)&c;
	
	tmp = ptr[0];
	ptr[0] = ptr[7];
	ptr[7] = tmp;
	tmp = ptr[1];
	ptr[1] = ptr[6];
	ptr[6] = tmp;
	tmp = ptr[2];
	ptr[2] = ptr[5];
	ptr[5] = tmp;
	tmp = ptr[3];
	ptr[3] = ptr[4];
	ptr[4] = tmp;
	
	return c;
}

