/*	XGArgStream.cpp
 *
 *		View factory argument stream. This is to encapsulate the
 *	parsing of the standard argument stream
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XDataUtil.h>
#include <XView.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGArgStream::XGArgStream
 *
 *		Create this argument stream from the void pointer provided
 */

XGArgStream::XGArgStream(void *data)
{
	fResData = data;
}

/*	XGArgStream::~XGArgStream
 *
 *		Make me go away
 */

XGArgStream::~XGArgStream()
{
}

/************************************************************************/
/*																		*/
/*	Internal accessor													*/
/*																		*/
/************************************************************************/

/*	XGArgStream::GetArg
 *
 *		Get this argument into the buffer provided
 */

void XGArgStream::GetArg(short len, void *data)
{
	register unsigned char *ptrsrc;
	register unsigned char *ptrdest;
	
	ptrsrc = (unsigned char *)fResData;
	ptrdest = (unsigned char *)data;
	while (len-- > 0) *ptrdest++ = *ptrsrc++;
	fResData = (void *)ptrsrc;
}

/************************************************************************/
/*																		*/
/*	Simple accessors													*/
/*																		*/
/************************************************************************/

/*	XGArgStream::GetBoolean
 *
 *		Get boolean argument
 */

bool XGArgStream::GetBoolean(void)
{
	unsigned short c;
	
	GetArg(sizeof(c),(void *)&c);
	return (bool)((c) ? true : false);
}

/*	XGArgStream::GetInteger
 *
 *		Get integer argument
 */

long XGArgStream::GetInteger(void)
{
	long l;
	
	GetArg(sizeof(l),(void *)&l);
	return l;
}

/*	XGArgStream::GetString
 *
 *		Get string. This is stored as a Pascal string with a length
 *	and the bytes of the string following
 */

void XGArgStream::GetString(short maxlen, char *data)
{
	unsigned char len;
	
	GetArg(sizeof(len),(void *)&len);	// Figure out length of string
	
	maxlen--;							// Make room for NULL terminator
	if (len < maxlen) maxlen = len;		// Get string actual length
	GetArg(maxlen,(void *)data);		// Get string
	data[maxlen] = 0;					// NULL terminate it
	
	if ((maxlen % 2) == 0) GetArg(1,(void *)&len);
}
