/*	XGView.cpp
 *
 *		This is the code for handling the drawing view.
 *
 *		Primarly, this code only contains the code necessary to handle
 *	the geometry of the view contents. The drawing support code is
 *	elsewhere.
 */
 
/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XCursorUtil.h>
#include <XApplication.h>
#include <XView.h>
#include <XWindow.h>

/************************************************************************/
/*																		*/
/*	Internal Routines													*/
/*																		*/
/************************************************************************/

/*	LAbs
 *
 *		So I don't need to import labs() from the standard C library
 */

static long LAbs(long x)
{
	return ((x < 0) ? -x : x);
}

/************************************************************************/
/*																		*/
/*	View Definition														*/
/*																		*/
/************************************************************************/

/*	XGView::XGView
 *
 *		Construct me. There are two ways to construct one of these things
 */

XGView::XGView(XGView *parent, XGSViewInitRecord &init, bool nowin)
{
	IInitView(parent,init,nowin);
}

XGView::XGView(XGView *parent, XGArgStream &s, bool nowin)
{
	XGSViewInitRecord init;
	
	/*
	 *	Pull the data and initialize me
	 */
	
	init.fViewType   = s.GetInteger();
	
	init.fViewID     = s.GetInteger();
	init.fRefNum     = s.GetInteger();

	init.fLockLeft   = s.GetBoolean();
	init.fLockTop    = s.GetBoolean();
	init.fLockRight  = s.GetBoolean();
	init.fLockBottom = s.GetBoolean();

	init.fAutoLeft   = s.GetInteger();
	init.fAutoTop    = s.GetInteger();
	init.fAutoRight  = s.GetInteger();
	init.fAutoBottom = s.GetInteger();

	init.fVisible    = s.GetBoolean();
	init.fEnabled    = s.GetBoolean();
	init.fTabStop    = s.GetBoolean();

	IInitView(parent,init,nowin);
}

XGView::XGView(XGArgStream &s)
{
	XGSViewInitRecord init;
	
	init.fViewType	 = s.GetInteger();	// strip type off the top
	init.fViewID     = -1;
	init.fRefNum     = 0;
	
	init.fLockLeft   = false;
	init.fLockTop    = false;
	init.fLockRight  = false;
	init.fLockBottom = false;
	
	init.fAutoLeft   = 0;
	init.fAutoTop    = 0;
	init.fAutoRight  = 100;			/* will change later... */
	init.fAutoBottom = 100;			/* but ain't empty now. */
	
	init.fVisible    = true;		/* View in window is visible */
	init.fEnabled    = true;
	init.fTabStop    = false;
	
	IInitView(NULL,init,true);
}

XGView::XGView(long v)
{
	XGSViewInitRecord init;
	
	init.fViewType	 = v;
	init.fViewID     = -1;
	init.fRefNum     = 0;
	
	init.fLockLeft   = false;
	init.fLockTop    = false;
	init.fLockRight  = false;
	init.fLockBottom = false;
	
	init.fAutoLeft   = 0;
	init.fAutoTop    = 0;
	init.fAutoRight  = 100;			/* will change later... */
	init.fAutoBottom = 100;			/* but ain't empty now. */
	
	init.fVisible    = true;		/* View in window is visible */
	init.fEnabled    = true;
	init.fTabStop    = false;
	
	IInitView(NULL,init,true);
}

/*	XGView::~XGView
 *
 *		Destroy me. This shuts this view down, kills the children,
 *	on Windows destroys the child window, and goes away
 */

XGView::~XGView()
{
	XGView *foo;
	
	/*
	 *	Invalidate the window (if it's still there)
	 */
	
#if OPT_WINOS == 1
	if (_GetHWND()) {
		::SetProp(_GetHWND(),_GViewClass,NULL);
		::DestroyWindow(_GetHWND());
	}
#endif
	
	/*
	 *	Invalidate my contents if I'm visible; forces the area
	 *	inside the window to be redrawn after I'm gone
	 */
	
	if (fParent) InvalView();
	
	/*
	 *	Remove me from my parent
	 */
	
	if (fParent) {
		if (fParent->fChild == this) {
			fParent->fChild = fSibling;
		} else {
			for (foo = fParent->fChild; foo != NULL; foo = foo->fSibling) {
				if (foo->fSibling == this) {
					foo->fSibling = fSibling;
					break;
				}
			}
		}
	}
	
	/*
	 *	Kill my children
	 */
	
	while (fChild) delete fChild;
}

/************************************************************************/
/*																		*/
/*	Public Event Messages												*/
/*																		*/
/************************************************************************/

/*	XGView::DoDrawView
 *
 *		Do the drawing in this view. By default this draws a stat box
 *	with some simple information
 */

void XGView::DoDrawView(Rect r)
{
	XGDraw draw(this);

#if OPT_MACOS == 1	
	unsigned char buffer[32];

	/*
	 *	Macintosh version to draw this
	 */
	
	::EraseRect(&r);
	
	r = GetContentRect();
	::FrameRect(&r);
	::InsetRect(&r,1,1);
	draw.ClipRect(&r);
	
	::TextFont(4);
	::TextSize(9);
	::MoveTo(3,12);
	buffer[0] = 4;
	buffer[1] = ((char *)&fViewType)[0];
	buffer[2] = ((char *)&fViewType)[1];
	buffer[3] = ((char *)&fViewType)[2];
	buffer[4] = ((char *)&fViewType)[3];
	::DrawString(buffer);
	
	::DrawString("\p:");
	::NumToString(fViewID,buffer);
	::DrawString(buffer);
#endif

#if OPT_WINOS == 1
	char buffer[32];

	/*
	 *	Windows version to draw this
	 */
	
	::FillRect(draw.GetDC(),&r,GetStockObject(WHITE_BRUSH));

	r = GetContentRect();
	::FrameRect(draw.GetDC(),&r,GetStockObject(BLACK_PEN));
	::InflateRect(&r,-1,-1);
	draw.ClipRect(&r);
	
	buffer[0] = ((char *)&fViewType)[3];
	buffer[1] = ((char *)&fViewType)[2];
	buffer[2] = ((char *)&fViewType)[1];
	buffer[3] = ((char *)&fViewType)[0];
	buffer[4] = ':';
	wsprintf(buffer+5,"%ld",(long)fViewID);
	::DrawText(draw.GetDC(),buffer,-1,&r,DT_SINGLELINE | DT_TOP | DT_NOPREFIX);
#endif
}

/*	XGView::DoActivate
 *
 *		By default this does nothing
 */

void XGView::DoActivate(bool)
{
}

/*	XGView::DoMouseDown
 *
 *		By default this does nothing
 */

bool XGView::DoMouseDown(Point,short)
{
	return false;
}

/*	XGView::DoMouseMove
 *
 *		By default this turns the cursor into the arrow
 */

void XGView::DoMouseMove(Point,short)
{
	SetCursorID(KNormalCursor);
}

/*	XGView::DoMouseUp
 *
 *		By default this does nothing
 */

void XGView::DoMouseUp(Point,short)
{
}

/*	XGView::DoSizeView
 *
 *		By default this calls the AutoLocate procedure to automatically
 *	locate the children inside of me
 */

void XGView::DoSizeView(void)
{
	XGView *foo;
	Rect r;
	
	r = GetContentRect();
	for (foo = fChild; foo != NULL; foo = foo->fSibling) {
		foo->AutoLocate(r);
	}
}

/*	XGView::DoMoveView
 *
 *		This handles when the view is moved globally in the global view.
 *	By default, does nothing
 */

void XGView::DoMoveView(void)
{
}

/************************************************************************/
/*																		*/
/*	Focus Support														*/
/*																		*/
/************************************************************************/

/*	XGView::SetFocus
 *
 *		Set the focus. On Windows, set the window focus here
 */

XGDispatch *XGView::SetFocus()
{
#if OPT_WINOS == 1
	if (fViewHWND) ::SetFocus(fViewHWND);
#endif

	return XGDispatch::SetFocus();
}

/************************************************************************/
/*																		*/
/*	Public Event Messages												*/
/*																		*/
/************************************************************************/

/*	XGView::ShowView
 *
 *		This shows this view if it was hidden
 */

void XGView::ShowView()
{
	if (!fVisible) {
#if OPT_WINOS == 1
		ShowWindow(fViewHWND,SW_SHOW);
#endif

		fVisible = true;
		InvalView();
	}
}

/*	XGView::HideView
 *
 *		Hide this view if it was visible
 */

void XGView::HideView()
{
	if (fVisible) {
		InvalView();
		fVisible = false;

#if OPT_WINOS == 1
		ShowWindow(fViewHWND,SW_HIDE);
#endif
	}
}

/*	XGView::EnableView
 *
 *		Enable this view. This is only a bit which is used for handling
 *	enable/disable states--this is a global but is only meaningful to
 *	children views of this view
 */

void XGView::EnableView()
{
	if (!fEnabled) {
		fEnabled = true;
		InvalView();

#if OPT_WINOS == 1
		EnableWindow(fViewHWND,true);
#endif
	}
}

/*	XGView::DisableView
 *
 *		Disable this view
 */

void XGView::DisableView()
{
	if (fEnabled) {
		fEnabled = false;
		InvalView();

#if OPT_WINOS == 1
		EnableWindow(fViewHWND,false);
#endif
	}
}

/*	XGView::IsActive
 *
 *		Is this view active? This is true if the window is active.
 *	So we ask the window
 */

bool XGView::IsActive() const
{
	if (fRoot) return fRoot->_IsActiveWindow();
	return false;
}

/*	XGView::IsCapture
 *
 *		Is this thing in the middle of a DoMouseDown()/DoMouseUp()
 *	thing? (That is, is the mouse currently being dragged in this view,
 *	and did we ask for dragging by returning true in the DoMouseDown()
 *	method?)
 */

bool XGView::IsCapture() const
{
	if (fRoot) return ((fRoot->_CapturedView() == this) ? true : false);
	return false;
}

/************************************************************************/
/*																		*/
/*	View Hierarchy Support												*/
/*																		*/
/************************************************************************/

/*	XGView::GetDocument
 *
 *		Get the document associated with this thing
 */

XGDocument *XGView::GetDocument() const
{
	return fRoot->_GetDocument();
}

/*	XGView::FindViewByID
 *
 *		This searches for a subview (or maybe this one?) which has this
 *	identifier. This recurses down until I find the subview which has
 *	this identifier.
 */

XGView *XGView::FindViewByID(short id)
{
	XGView *v;
	XGView *r;
	
	if (fViewID == id) return this;
	for (v = fChild; v != NULL; v = v->fSibling) {
		r = v->FindViewByID(id);
		if (r) return r;
	}
	return NULL;
}

/*	XGView::FindViewByPoint
 *
 *		Given a *global* point (one in the coordinate system of the 
 *	containing window), this finds the view that contains this point.
 *
 *		This recurses down until I find the smallest view which contains
 *	this point.
 */

XGView *XGView::FindViewByPoint(Point pt)
{
	XGView *v;
	XGView *r;
	
	if (!PtInRect(pt,&fGlobalRect)) return NULL;
	for (v = fChild; v != NULL; v = v->fSibling) {
		r = v->FindViewByPoint(pt);
		if (r) return r;
	}
	return this;
}

/************************************************************************/
/*																		*/
/*	Scrolling Support													*/
/*																		*/
/************************************************************************/

/*	XGView::Scroll
 *
 *		This is the method which is called when it's time to scroll this
 *	thing, such as processing scrollbar messages. By default, this scrolls
 *	the entire thing.
 */

void XGView::Scroll(long x, long y)
{
	ScrollView(x,y);
}

/*	XGView::ScrollView
 *
 *		This handles the scrolling view itself. This is a bit tricky, given
 *	the various operating systems this thing can be hosted on. Basically,
 *	I can scroll the bits in this view without having a drawing context
 *	open to this view. This is accomplished with various black magic
 *	incantations.
 */

void XGView::ScrollView(long x, long y, Rect *r)
{
	Rect s;
	
	/*
	 *	First, in an OS independant way, find the area that actually
	 *	gets scrolled
	 */
	
	if (r == NULL) s = GetContentRect();
	else s = *r;
	ViewToGlobal(&s);
	if (!::SectRect(&fGlobalRect,&s,&s)) return;	/* Not visible */
	
	/*
	 *	Determine if the amount we get scrolled is too wide or tall.
	 *	If it is, simply invalidate the entire view
	 */
	
	if (((s.right - s.left) <= LAbs(x)) || ((s.bottom - s.top) <= LAbs(y))) {
		InvalView();
		return;
	}
	
	/*
	 *	We have to scroll. Oh, well. At least do it with a little class...
	 */
	
#if OPT_MACOS == 1
	_PushDC();
	RgnHandle rgn;
	
	rgn = ::NewRgn();
	::ScrollRect(&s,(short)x,(short)y,rgn);
	::InvalRgn(rgn);
	::DisposeRgn(rgn);
	
	_PopDC();
#endif

#if OPT_WINOS == 1
	::ScrollWindowEx(_GetHWND(),(int)x,(int)y,&s,&s,NULL,NULL,SW_ERASE | SW_INVALIDATE);
#endif
}

/************************************************************************/
/*																		*/
/*	View Location and Manipulation										*/
/*																		*/
/************************************************************************/

/*	XGView::GetContentRect
 *
 *		Get the content rect. This is a cheat; this is just a wrapper
 *	to get the local rectangle size
 */

Rect XGView::GetContentRect() const
{
	Rect r;
	Point pt;
	
	pt = GetSize();
	r.left = 0;
	r.top = 0;
	r.right = pt.h;
	r.bottom = pt.v;
	
	return r;
}

/*	XGView::SetLocation
 *
 *		This updates the location of this thing. This takes the cheap way
 *	out in moving my view: this simply invalidates the old and the new
 *	regions where the view is.
 *
 *		The rectangle is specified in the local coordinates of the parent
 *	view.
 */

void XGView::SetLocation(Rect r)
{
	Point newSize;
	Point newPos;
	Point oldSize;
	Point oldPos;
	
	/*
	 *	Calculate the new position and size
	 */
	
	newPos.h  = r.left;
	newPos.v  = r.top;
	newSize.h = r.right - r.left;
	newSize.v = r.bottom - r.top;
	
	/*
	 *	Determine if this view even moves; if it doesn't, this
	 *	aborts this operation.
	 */
	
	if ((newPos.h == fPos.h) && (newPos.v == fPos.v) &&
			(newSize.h == fSize.h) && (newSize.v == fSize.v)) return;
			
	/*
	 *	We're about to move this thing. First, invalidate the old
	 *	area where this thing used to be.
	 */
	
	InvalView();
	
	/*
	 *	Now, calculate this view's positioning information
	 */
	
	fGlobalRect = r;
	if (fParent) {
		fParent->ViewToGlobal(&fGlobalRect);
		SectRect(&fGlobalRect,&(fParent->fGlobalRect),&fGlobalRect);
	}
	oldSize = fSize;
	oldPos = fPos;
	
	fPos = newPos;
	fSize = newSize;
	fGlobalPos = newPos;
	if (fParent) {
		fParent->ViewToGlobal(&fGlobalPos);
	}
	
	/*
	 *	Reposition the container window (if there is one: WinOS only)
	 */
	
#if OPT_WINOS == 1
	if (this != fRoot) {
		/*
		 *	This skips the following if we are the window. That's because
		 *	generally the XGWindow class calls the SetLocation() method
		 *	when it is resized. That resize only happens when the window
		 *	has already been resized--which means we are already the right
		 *	size to begin with.
		 */
		
		::SetWindowPos(fViewHWND,NULL,r.left,r.top,fSize.h,fSize.v,
				SWP_NOZORDER | SWP_NOREDRAW);
	}
#endif

	/*
	 *	Invalidate this thing again
	 */
	
	InvalView();

	/*
	 *	And send the callbacks
	 */
	
	if ((oldPos.h != fPos.h) || (oldPos.v != fPos.v)) CalcPos();
	if ((oldSize.h != fSize.h) || (oldSize.v != fSize.v)) DoSizeView();
}

/*	XGView::AutoLocate
 *
 *		This handles automatically relocating this view when the DoSizeView
 *	call is made.
 */

void XGView::AutoLocate(Rect r)
{
	if (fParent == NULL) return;		// Can't autolocate root view
	
	/*
	 *	Calculate the location of this thing and move the view
	 */
	
	SetLocation(CalcAutoLocate(r));
}

/************************************************************************/
/*																		*/
/*	Drawing Support														*/
/*																		*/
/************************************************************************/

/*	XGView::InvalView
 *
 *		Invalidate the view
 */

void XGView::InvalView(Rect *r)
{
#if OPT_MACOS == 1
	Rect s;
	
	if (r == NULL) {
		_PushDC();
		::InvalRect(&fGlobalRect);
		_PopDC();
	} else {
		s = *r;
		ViewToGlobal(&s);						// Convert to global coordinates
		if (::SectRect(&s,&fGlobalRect,&s)) {	// Clip against my local stuff
			_PushDC();
			::InvalRect(&s);					// Invalidate this area
			_PopDC();
		}
	}
#endif

#if OPT_WINOS == 1
	InvalidateRect(fViewHWND,r,false);
#endif
}

/*	XGView::ValidView
 *
 *		Validate this view. Like the above
 */

void XGView::ValidView(Rect *r)
{
#if OPT_MACOS == 1
	Rect s;
	
	if (r == NULL) {
		_PushDC();
		::ValidRect(&fGlobalRect);
		_PopDC();
	} else {
		s = *r;
		ViewToGlobal(&s);						// Convert to global coordinates
		if (::SectRect(&s,&fGlobalRect,&s)) {	// Clip against my local stuff
			_PushDC();
			::ValidRect(&s);					// Invalidate this area
			_PopDC();
		}
	}
#endif

#if OPT_WINOS == 1
	ValidateRect(fViewHWND,r);
#endif
}

/*	XGView::UpdateView
 *
 *		Update this view if this view is not valid.
 */

void XGView::UpdateView()
{
#if OPT_WINOS == 1
	UpdateWindow(fViewHWND);
#endif

#if OPT_MACOS == 1
	RgnHandle rgn;
	Point pt;
	RgnHandle tmp;
	Rect r;
	GrafPtr port;
	
	if (!fVisible) return;
	
	/*
	 *	Initialize stuff
	 */
	
	XGDraw draw(this,false);			// Set this as current port
	::GetPort(&port);
	pt.h = 0, pt.v = 0;
	::GlobalToLocal(&pt);				// Shift to window coordinates
	
	/*
	 *	Calculate the intersection of the update region with the
	 *	view rectangle
	 */
	
	rgn = ::NewRgn();					// Get the update region
	::CopyRgn(((WindowPeek)port)->updateRgn,rgn);
	::OffsetRgn(rgn,pt.h,pt.v);			// Offset the region to window coords
	tmp = ::NewRgn();
	r = GetContentRect();
	::OffsetRect(&r,fGlobalPos.h,fGlobalPos.v);
	::RectRgn(tmp,&r);
	::SectRgn(rgn,tmp,rgn);				// Intersect with view rectangle
	::DisposeRgn(tmp);
	
	if (!EmptyRgn(rgn)) {
		/*
		 *	This is a bit wierd, but what I do is to set the invalid
		 *	region to the region that I calculated, force a normal
		 *	validation (BeginUpdate/EndUpdate) cycle, and then
		 *	invalidate the area not in the region I calculated...
		 */
		
		tmp = ::NewRgn();
		::CopyRgn(((WindowPeek)port)->updateRgn,tmp);
		::OffsetRgn(tmp,pt.h,pt.v);		// Offset to window coordinates
		::DiffRgn(tmp,rgn,tmp);			// Area remaining (window coords)
		
		r = (**rgn).rgnBBox;			// Get bounding rectangle
		::OffsetRgn(rgn,-pt.h,-pt.v);	// Move my invalid area to global cds
		::CopyRgn(rgn,((WindowPeek)port)->updateRgn);	// Replace area
		
		::BeginUpdate(port);			// Update this window
		_DrawViewHierarchy(r);
		::EndUpdate(port);
		
		::SetPort(port);
		::SetOrigin(0,0);				// Move to window coordinates
		::InvalRgn(tmp);				// Invalidate the remaining area
		::DisposeRgn(tmp);
	}
		
	::DisposeRgn(rgn);					// Clean up and exit
#endif
}

/************************************************************************/
/*																		*/
/*	Magical Internal Stuff												*/
/*																		*/
/************************************************************************/

#if OPT_WINOS == 1

/*	XGView::_CreateHWND
 *
 *		Create the child window which represents my view. By default this
 *	builds a child window of my class; controls can be created by overloading
 *	this thing
 */

HWND XGView::_CreateHWND(Rect r)
{
	HWND w;
	
	w = ::CreateWindow(_GViewClass,					// View class type
					   "",							// Title (empty)
					   WS_CHILD, //| WS_CLIPCHILDREN,	// window type
					   r.left,
					   r.top,
					   r.right-r.left,
					   r.bottom-r.top,				// location of this window
					   GetParent()->_GetHWND(),
					   NULL,
					   _GInstance,
					   NULL);
	return w;
}

/*	XGView::_GetXGView
 *
 *		Get the XGView associated with this window. This knows that a
 *	window class may be stored under the window property instead of the
 *	view property.
 */

XGView *XGView::_GetXGView(HWND h)
{
	XGView *v;
	XGWindow *w;
	
	v = (XGView *)::GetProp(h,_GViewClass);
	if (v == NULL) {
		w = (XGWindow *)::GetProp(h,_GWindowClass);
		return w;
	} else return v;
}

#endif /* OPT_WINOS */


#if OPT_MACOS == 1

/*	XGView::_DrawViewHierarchy
 *
 *		Scan through the hierarchy and figure out which views are
 *	visible and can be drawn
 */

void XGView::_DrawViewHierarchy(Rect ginval)
{
	Rect r;
	Rect s;
	XGView *v;
	
	/*
	 *	Am I visible?
	 */
	
	if (!fVisible) return;				// Nope.
	
	/*
	 *	Am I even in the invalid area?
	 */
	
	if (::SectRect(&ginval,&fGlobalRect,&r)) {
		/*
		 *	Do me first
		 */
		
		s = r;
		GlobalToView(&s);
		DoDrawView(s);
		
		/*
		 *	Now let my children draw inside of me
		 */
		
		for (v = fChild; v != NULL; v = v->fSibling) {
			v->_DrawViewHierarchy(r);
		}
		
	}
}

/*	XGView::_ActivateViewHierarchy
 *
 *		Run through and activate or deactivate all the views
 */

void XGView::_ActivateViewHierarchy(bool active)
{
	XGView *v;
	
	for (v = fChild; v != NULL; v = v->fSibling) {
		v->_ActivateViewHierarchy(active);
	}
	DoActivate(active);
}

#endif /* OPT_MACOS */

/************************************************************************/
/*																		*/
/*	Very magical stuff													*/
/*																		*/
/************************************************************************/

#if OPT_MACOS == 1

static GrafPtr		_GOldPort;
static GrafPtr		_GNewPort;
static RgnHandle	_GOldClip;
static Point		_GOldScroll;

/*	XGView::_PushDC
 *
 *		What this does is to set the drawing port that contains this
 *	view as the current, unclipped, unmodified port rectangle
 */

void XGView::_PushDC(void)
{
	Rect r;
	
	/*
	 *	What was there before?
	 */
	
	::GetPort(&_GOldPort);
	
	/*
	 *	Get where we are drawing now
	 */
	
	_GNewPort = fRoot->_GetGrafPtr();
	
	/*
	 *	Save the port's scroll and clip offsets
	 */
	
	::SetPort(_GNewPort);
	r = _GNewPort->portRect;
	_GOldScroll.h = r.left;
	_GOldScroll.v = r.top;
	_GOldClip = NewRgn();
	::GetClip(_GOldClip);
	
	/*
	 *	Now set this correctly
	 */
	
	::SetOrigin(0,0);
	::ClipRect(&r);
}

/*	XGView::_PopDC
 *
 *		Pop this device context back to what was going on before
 */

void XGView::_PopDC()
{
	/*
	 *	Restore the current drawing port to the prior clipping and
	 *	offset settings
	 */
	
	::SetPort(_GNewPort);
	::SetOrigin(_GOldScroll.h,_GOldScroll.v);
	::SetClip(_GOldClip);
	DisposeRgn(_GOldClip);
	
	/*
	 *	And move back to the prior port
	 */
	
	::SetPort(_GOldPort);
}

/*	XGView::_CalcInval
 *
 *		Calculate the global bounding rectangle of the invalidated region
 */

Rect XGView::_CalcInval()
{
	GrafPtr port;
	RgnHandle rgn;
	
	port = fRoot->_GetGrafPtr();
	rgn = ((WindowPeek)port)->updateRgn;
	return (**rgn).rgnBBox;
}

#endif /* OPT_MACOS */

/************************************************************************/
/*																		*/
/*	Private initialization												*/
/*																		*/
/************************************************************************/

/*	XGView::IInitView
 *
 *		This is the primary initialization routine for this view.
 */

void XGView::IInitView(XGView *parent, XGSViewInitRecord &init, bool nowin)
{
#if OPT_MACOS == 1
	#pragma unused(nowin)
#endif

	Rect s;

	/*
	 *	Initialize the basic hierarchy
	 */
	
	fChild = NULL;
	if (parent) {
		// Hook me into the parent
		fRoot = parent->fRoot;
		fParent = parent;
		fSibling = parent->fChild;
		parent->fChild = this;
	} else {
		// Don't hook me into the parent--I *am* the root
		fParent = NULL;
		fSibling = NULL;
		fRoot = (XGWindow *)this;			/* Me--I'm my own root */
	}
	
	/*
	 *	Get the view specification parameters
	 */
	
	fViewType   = init.fViewType;
	fViewID     = init.fViewID;
	fRefNum     = init.fRefNum;
	
	/*
	 *	Get the size of this thing
	 */
	
	fLockLeft   = init.fLockLeft;
	fLockTop    = init.fLockTop;
	fLockRight  = init.fLockRight;
	fLockBottom = init.fLockBottom;
	
	fAutoLeft   = init.fAutoLeft;
	fAutoTop    = init.fAutoTop;
	fAutoRight  = init.fAutoRight;
	fAutoBottom = init.fAutoBottom;
	
	/*
	 *	Calculate the size of this thing based on the auto parameters
	 */
	
	if (fParent) {
		s = CalcAutoLocate(fParent->GetContentRect());
	} else {
		s.left = fAutoLeft;
		s.right = fAutoRight;
		s.top = fAutoTop;
		s.bottom = fAutoBottom;
	}
	
	/*
	 *	Initialize the view positioning information
	 */
	
	fSize.h = s.right - s.left;
	fSize.v = s.bottom - s.top;
	fPos.h = s.left;
	fPos.v = s.top;
	fGlobalPos = fPos;
	fGlobalRect = s;
	if (fParent) {
		fParent->ViewToGlobal(&fGlobalPos);
		fParent->ViewToGlobal(&fGlobalRect);
		
		::SectRect(&(fParent->fGlobalRect),&fGlobalRect,&fGlobalRect);
	}
	
	/*
	 *	Initialization
	 */
	
	fVisible    = init.fVisible;
	fEnabled    = init.fEnabled;
	fTabStop    = init.fTabStop;
	
	/*
	 *	Initialize the window in this view (if there is any) -- only if
	 *	this is not a window. (Windows are handled elsewhere)
	 */
	
#if OPT_WINOS == 1
	if (!nowin) {
		_GView = this;
		fViewHWND = _CreateHWND(s);
		if (fViewHWND) {
			SetProp(fViewHWND,_GViewClass,(HANDLE)this);
			if (fVisible) ShowWindow(fViewHWND,SW_SHOW);
		}
		_GView = NULL;
	}
#endif
}

/*	XGView::CalcPos
 *
 *		Calculate the position of the children views
 */

void XGView::CalcPos(void)
{
	Point newPos;
	XGView *v;
	
	/*
	 *	Run through the children, and move them. If they do move, then
	 *	we move them, and recurse
	 */
	
	for (v = fChild; v != NULL; v = v->fSibling) {
		newPos.h = fGlobalPos.h + v->fPos.h;
		newPos.v = fGlobalPos.v + v->fPos.v;
		if ((newPos.h != v->fGlobalPos.h) || (newPos.v != v->fGlobalPos.v)) {
			v->fGlobalPos = newPos;
			v->CalcPos();
		}
	}
	
	/*
	 *	Let me know we moved
	 */
	
	DoMoveView();
}

/*	XGView::CalcAutoLocate
 *
 *		Given the parameters for how this view should be locked to my parent's
 *	view, this calculates the various location parameters
 */

Rect XGView::CalcAutoLocate(Rect r)
{
	Rect s;
	
	if (fLockLeft) s.left = r.left + fAutoLeft;
	else s.left = r.right + fAutoLeft;
	if (!fLockRight) s.right = r.left + fAutoRight;
	else s.right = r.right + fAutoRight;
	
	if (fLockTop) s.top = r.top + fAutoTop;
	else s.top = r.bottom + fAutoTop;
	if (!fLockBottom) s.bottom = r.top + fAutoBottom;
	else s.bottom = r.bottom + fAutoBottom;

	return s;
}

