/*	XGDocument.cpp
 *
 *		The basic document processing class
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#include <XDocument.h>

/************************************************************************/
/*																		*/
/*	Construction/Destruction											*/
/*																		*/
/************************************************************************/

/*	XGDocument::XGDocument
 *
 *		Create this
 */

XGDocument::XGDocument()
{
}

/*	XGDocument::~XGDocument
 *
 *		Delete this
 */

XGDocument::~XGDocument()
{
}

/************************************************************************/
/*																		*/
/*	Dispatch Support													*/
/*																		*/
/************************************************************************/

/*	XGDocument::ReceiveDispatch
 *
 *		Handle the save/restore menu items
 */

long XGDocument::ReceiveDispatch(long msg, long arg, void *parg)
{
	XGSMenuStatusRecord *gm;
	
	switch (msg) {
		case KEventDoMenuCommand:
			/*
			 *	Handle the following menu commands:
			 *
			 *		Quit/Close/About
			 *		Undo/Cut/Copy/Paste/Clear (desk accessories)
			 */
			
			if (arg == KMenuFileSave) {
				if (IsDirty()) {
					DoSave();
					return 1;
				}
			}
			if (arg == KMenuFileSaveAs) {
				DoSaveAs();
				return 1;
			}
			if (arg == KMenuFileRevert) {
				if (IsDirty() && IsNamed()) {
					DoRevert();
					return 1;
				}
			}
			break;

		case KEventGetMenuStatus:
			/*
			 *	Handle enabling those I can handle
			 */
			
			gm = (XGSMenuStatusRecord *)parg;
			if (arg == KMenuFileSaveAs) {
				gm->enable = true;
				return 1;
			}
			if (arg == KMenuFileSave) {
				if (IsDirty()) gm->enable = true;
				return 1;
			}
			if (arg == KMenuFileRevert) {
				if (IsDirty() && IsNamed()) gm->enable = true;
				return 1;
			}
			break;
	}
	
	return XGDispatch::ReceiveDispatch(msg,arg,parg);
}
