/*	XError.h
 *
 *		This contains the error processing declarations
 */
 
/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XERROR_H__
#define __XERROR_H__

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif

/************************************************************************/
/*																		*/
/*	Error declarations													*/
/*																		*/
/************************************************************************/

/*	XError
 *
 *		This is the basic error object. This is the default error type
 *	of all my internally thrown errors, and provides the standard
 *	callbacks for dealing with 'em
 *
 *		I assume that when one of these things are thrown, this
 *	will be caught just outside my event handler loop. The event
 *	handler loop contains a callback which then knows how to present
 *	the error to the user
 */

class XError {
	public:
								XError();
		virtual					~XError();
		
		virtual void			DisplayError(void) = 0;
};

/*	XAbort
 *
 *		This is an abort error. This throws without causing so much
 *	as a peep, and is assumed to be thrown only when the user cancels
 *	an operation or another
 */

class XAbort : public XError {
	public:
								XAbort();
		virtual					~XAbort();
		
		virtual void			DisplayError(void);
};

/*	XPostError
 *
 *		This is the routine for posting an error that consists of
 *	a text string which contains the notification.
 */

class XPostError : public XError {
	public:
								XPostError(char *msg,...);
		virtual					~XPostError();
		
		virtual void			DisplayError(void);
	private:
		char					fBuffer[80];
};

/************************************************************************/
/*																		*/
/*	Assertion testing													*/
/*																		*/
/************************************************************************/

#define XASSERT(x)	{ if (!(x)) throw XPostError("ASSERT: '%s' at %s@%ld", \
							#x,__FILE__,(long)__LINE__); }

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif /* __XERROR_H__ */
