/*	XStdButtons.h
 *
 *		Standard buttons
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XSTDBUTTONS_H__
#define __XSTDBUTTONS_H__

#include <XView.h>
#include <XEvent.h>

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif

/************************************************************************/
/*																		*/
/*	Internal structures													*/
/*																		*/
/************************************************************************/

/*	XGSButtonInitRecord
 *
 *		This contains the common initialization parameters to initialize
 *	a button
 */

struct XGSButtonInitRecord {
	XGSViewInitRecord			v;
	char						title[256];
};

/************************************************************************/
/*																		*/
/*	Button Events														*/
/*																		*/
/************************************************************************/

/*	KEventButton
 *
 *		The specified button was clicked.
 *
 *			arg			The ViewID of the button clicked
 *			parg		Pointer to the button view class
 */

#define KEventButton			'bclk'

/************************************************************************/
/*																		*/
/*	Standard Controls													*/
/*																		*/
/************************************************************************/

/*	XGStdButton
 *
 *		This is the root button control; this contains the common
 *	treatment of system controls.
 */

class XGStdButton : public XGView {
	public:
								XGStdButton(XGView *parent, XGArgStream &init, bool f = false);
								XGStdButton(XGView *parent, XGSViewInitRecord &, bool f = false);
		virtual					~XGStdButton();
		
		/*
		 *	Standard behaviour routines
		 */
		
		void					DoDrawView(Rect);
		void					DoActivate(bool);
		bool					DoMouseDown(Point,short);
		void					DoMoveView(void);
		void					DoSizeView(void);
		
		void					ShowView();
		void					HideView();
		void					EnableView();
		void					DisableView();

		/*
		 *	Standard messages
		 */
		
		long					ReceiveDispatch(long,long,void *);

		/*
		 *	Value setting routines
		 */
		
		virtual short			GetValue();
		virtual void			SetValue(short);
		virtual void			GetText(char *);
		virtual void			SetText(char *);
		
	protected:
#if OPT_MACOS == 1
		short					fFont;
		short					fSize;
		ControlHandle			fControl;
#endif
};

/*	XGStdPushButton
 *
 *		Standard push button.
 */

class XGStdPushButton : public XGStdButton {
	public:
								XGStdPushButton(XGView *parent, XGArgStream &init);
								XGStdPushButton(XGView *parent, XGSButtonInitRecord &init);
		virtual					~XGStdPushButton();
	private:
		void					Init(char *title); 
};

/*	XGStdRadioButton
 *
 *		Standard radio button
 */

class XGStdRadioButton : public XGStdButton {
	public:
								XGStdRadioButton(XGView *parent, XGArgStream &init);
								XGStdRadioButton(XGView *parent, XGSButtonInitRecord &init);
		virtual					~XGStdRadioButton();
	private:
		void					Init(char *title); 
};

/*	XGStdCheckButton
 *
 *		Check button
 */

class XGStdCheckButton : public XGStdButton {
	public:
								XGStdCheckButton(XGView *parent, XGArgStream &init);
								XGStdCheckButton(XGView *parent, XGSButtonInitRecord &init);
		virtual					~XGStdCheckButton();
	private:
		void					Init(char *title); 
};


#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif // __XSTDBUTTONS_H__
