/*	XStdList.h
 *
 *		Standard text controls
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XSTDLIST_H__
#define __XSTDLIST_H__

#include <XView.h>
#include <XEvent.h>

#if OPT_MACOS == 1
#include <Icons.h>
#endif

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif


/************************************************************************/
/*																		*/
/*	Internal structures													*/
/*																		*/
/************************************************************************/

/*	XGSImageInitRecord
 *
 *		This contains the common initialization parameters to initialize
 *	a button
 */

struct XGSListInitRecord {
	XGSViewInitRecord			v;
	bool						msel;
};

/************************************************************************/
/*																		*/
/*	Messages															*/
/*																		*/
/************************************************************************/

/*	KEventListSelect
 *
 *		The selection has changed on the specified list
 *
 *			arg			The ViewID of the view which changed
 *			parg		Pointer to the (XGStdListBox *) view
 */

#define KEventListSelect		'lsel'

/*	KEventListDblClk
 *
 *		The selection has been double-clicked
 *
 *			arg			The ViewID of the view which changed
 *			parg		Pointer to the (XGStdList *) view
 */

#define KEventListDblClk		'lclk'

/*	KEventMenuSelect
 *
 *		The menu has changed selection
 *
 *			arg			The ViewID of the view which changed
 *			parg		Pointer to the (XGStdPopupMenu *) view
 */

#define KEventMenuSelect		'msel'

/************************************************************************/
/*																		*/
/*	Standard Classes													*/
/*																		*/
/************************************************************************/

/*	XGStdList
 *
 *		This is the core list handling class. This handles inserting
 *	and deleting items from a list of items.
 */

class XGStdList : public XGView {
	public:
								XGStdList(XGView *parent, XGArgStream &init, bool f = false);
								XGStdList(XGView *parent, XGSViewInitRecord &init, bool f = false);
		virtual					~XGStdList();
		
		/*
		 *	List access
		 */
		
		virtual long			Length() const = 0;
		virtual void			Insert(long pos, const char *) = 0;
		virtual void			Delete(long pos) = 0;
		virtual void			Get(long,char *) const = 0;
		virtual void			Set(long,const char *) = 0;
									
		void					operator += (const char *s) 
									{
										Insert(Length(),s);
									}
};

/*	XGStdListBox
 *
 *		This runs a list box. On the Mac, I use the list manager;
 *	on Windows, I use the list box control.
 */

class XGStdListBox : public XGStdList {
	public:
								XGStdListBox(XGView *parent, XGArgStream &init);
								XGStdListBox(XGView *parent, XGSListInitRecord &init);
		virtual					~XGStdListBox();
		
		/*
		 *	Content modification
		 */
		
		virtual long			Length() const;
		virtual void			Insert(long pos, const char *);
		virtual void			Delete(long pos);
		virtual void			Get(long,char *) const;
		virtual void			Set(long,const char *);
		
		/*
		 *	Selection routines
		 */
		
		long					GetSelect(void) const;
		bool					IsSelect(long) const;
		void					SetSelect(long,bool);
		void					ClearSelect(void);
		
		/*
		 *	Standard events
		 */
		
		void					DoDrawView(Rect);
		void					DoActivate(bool);
		bool					DoMouseDown(Point,short);
		void					DoSizeView();
		void					DoMoveView();

	private:
		void					Init(bool);
		bool					fMultiSelect;
		
#if OPT_MACOS == 1
		short					fFont;
		short					fSize;
		ListHandle				fList;
#endif
};


#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif
