/*	XStdScroll.h
 *
 *		Standard scroll bar
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XSTDSCROLL_H__
#define __XSTDSCROLL_H__

#include <XView.h>
#include <XEvent.h>

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif

/************************************************************************/
/*																		*/
/*	Internal structures													*/
/*																		*/
/************************************************************************/

/*	XGSScrollInitRecord
 *
 *		This contains the common initialization parameters to initialize
 *	a button
 */

struct XGSScrollInitRecord {
	XGSViewInitRecord			v;
	short						page;
	short						min;
	short						max;
	short						val;
};

/************************************************************************/
/*																		*/
/*	Button Events														*/
/*																		*/
/************************************************************************/

/*
 *	A note about the two scrolling events below.
 *
 *		These two events are designed to operate in conjunction with
 *	eachother. Generally, when the scrollbar is modified, both messages
 *	are sent. This is to give the view a chance to completely update
 *	it's bits before moving the bits around.
 *
 *		Thus, it is expected that a view implementing these two
 *	messages will do the following
 *
 *		On KEventPreScroll:
 *
 *			The scrollbar is at it's old value.
 *			View updates it's view by calling UpdateView()
 *
 *		On KEventScroll:
 *			The scrollbar is at it's new value; old value provided
 *			View moves the bits by calling ScrollView()
 *			ScrollView invalidates the bits that are scrolled from
 *				offscreen.
 *
 *		Note that when the scrollbar is repeatedly updated (such as
 *	when you hold the 'up' button down), both messages are sent
 *	repeatedly one after the other.
 *
 *		And note that normally both messages will be sent--you will
 *	normally never get one message or the other but not both. And
 *	by 'normally' I mean that if it happens, the system is so screwed
 *	up, improperly updated bits is the least of the user's problems!
 */

/*	KEventScroll
 *
 *		The scroll bar was modified. This gives the view being scrolled
 *	the opportunity to move it's bits.
 *
 *			arg			The ViewID of the updated scroll bar
 *			parg		Points to structure given below
 */

#define KEventScroll			'scrl'

struct XGSScrollEvent {
	class XGStdScroll			*scroll;
	short						oldvalue;
};

/*	KEventPreScroll
 *
 *		The scroll bar is about to be modified. This is called when
 *	the scroll bar is about to have it's value changed, and gives
 *	a view being scrolled the opportunity to refresh the bits before
 *	moving those bits around.
 *
 *		This fixes a bug with scrolling a view which needs to be
 *	updated--this allows the scrolling region the chance to simply
 *	do an UpdateView(). That's because most OS GUI environments don't
 *	cope with scrolling invalid bits and refreshing them correctly
 *
 *			arg			The ViewID of the scroll bar being updated
 *			parg		Points to the XGStdScroll being updated
 */

#define KEventPreScroll			'pscr'

/************************************************************************/
/*																		*/
/*	Standard Controls													*/
/*																		*/
/************************************************************************/

/*	XGStdScroll
 *
 *		This is the scroll control.
 */

class XGStdScroll : public XGView {
	public:
								XGStdScroll(XGView *parent, XGArgStream &init);
								XGStdScroll(XGView *parent, XGSScrollInitRecord &init);
		virtual					~XGStdScroll();
		
		/*
		 *	Standard behaviour routines
		 */
		
		void					DoDrawView(Rect);
		void					DoActivate(bool);
		bool					DoMouseDown(Point,short);
		void					DoSizeView(void);
		void					DoMoveView(void);
		
		void					ShowView();
		void					HideView();
		void					EnableView();
		void					DisableView();
		
		/*
		 *	Standard messages
		 */
		
		long					ReceiveDispatch(long,long,void *);

		/*
		 *	Value setting routines
		 */
		
		virtual short			GetValue();
		virtual void			SetValue(short);
		virtual short			GetMinValue();
		virtual short			GetMaxValue();
		virtual short			GetPageValue();
		virtual void			SetMinValue(short);
		virtual void			SetMaxValue(short);
		virtual void			SetPageValue(short);

	protected:
		short					fPageValue;

#if OPT_MACOS == 1
		static ControlActionUPP	gProc;
		static pascal void		ActionProc(ControlHandle,short);
		ControlHandle			fControl;
#endif

	private:
		void					Init(short,short,short,short);
};


#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif // __XSTDSCROLL_H__
