/*	XStdScrollView.h
 *
 *		Standard scrolling view
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XSTDSCROLLVIEW_H__
#define __XSTDSCROLLVIEW_H__

#include <XView.h>
#include <XEvent.h>
#include <XStdScroll.h>

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif

/************************************************************************/
/*																		*/
/*	Internal structures													*/
/*																		*/
/************************************************************************/

/*	XGSScrollViewInitRecord
 *
 *		This contains the common initialization parameters to initialize
 *	a button
 */

struct XGSScrollViewInitRecord {
	XGSViewInitRecord			v;
	bool						hscroll;
	bool						vscroll;
	bool						corner;
	short						viewID;
};

/************************************************************************/
/*																		*/
/*	Scrolling Engine													*/
/*																		*/
/************************************************************************/

/*	XGScrollView
 *
 *		This is the scrolling view
 */

class XGScrollView : public XGView {
	public:
								XGScrollView(XGView *, XGArgStream &);
								XGScrollView(XGView *, XGSScrollViewInitRecord &);
		virtual					~XGScrollView();
		
		/*
		 *	Display events
		 */
		
		void					DoDrawView(Rect);
		void					DoSizeView();
		
		/*
		 *	scrolling support routines
		 */
		
		void					GetScrollValue(short *, short *);
		void					SetScrollValue(short,short);
		void					GetScrollMax(short *, short *);
		void					SetScrollMax(short,short);
		void					GetScrollPage(short *, short *);
		void					SetScrollPage(short,short);
		short					GetScrollDelta()
									{
										return fDelta;
									}
		void					SetScrollDelta(short x)
									{
										x = fDelta;
										InvalChildView();
									}

		/*
		 *	Message dispatch
		 */
		
		long					ReceiveDispatch(long,long,void *);
		
	private:
		void					Init(bool h, bool v, bool c, short s);

		Rect					CalcHScroll();
		Rect					CalcVScroll();
		Rect					CalcCorner();
		Rect					CalcScroll();
		
		void					InvalChildView();
		void					UpdateChildView();
		void					ScrollChildView(short ox, short oy);
		
		short					fChildID;
		short					fDelta;
		bool					fCorner;
		bool					fBounce;
		XGStdScroll				*fHScroll;
		XGStdScroll				*fVScroll;
};

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif // __XSTDSCROLL_H__
