/*	XStdText.h
 *
 *		Standard text controls
 */

/*  YAAF - Yet another application framework
 *  Copyright (C) 1997 William Edward Woody and In Phase Consulting
 *  
 *  This library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU Library
 *  General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or any
 *  later version.
 *  
 *  This library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied
 *  warranty of MERCHANTABIILITY or FITNESS FOR A PARTICULAR
 *  PURPOSE. See the GNU Library General Public License for
 *  more details.
 *  
 *  You should have received a copy of the GNU Library General
 *  Public License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 *  
 *  To contact the author, either e-mail me at
 *  woody@alumni.caltech.edu, or write to us at
 *  
 *          William Edward Woody
 *          In Phase Consulting
 *          1545 Ard Eevin Avenue
 *          Glendale, CA 91202
 */

#ifndef __XSTDTEXT_H__
#define __XSTDTEXT_H__

#include <XView.h>
#include <XEvent.h>

#if OPT_MACOS == 1
#include <XString.h>
#endif

#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=power
	#endif
	#if defined(__INTEL__)
		#pragma pack(push,2)
	#endif
#endif


/************************************************************************/
/*																		*/
/*	Internal structures													*/
/*																		*/
/************************************************************************/

/*	XGSTextInitRecord
 *
 *		This contains the common initialization parameters to initialize
 *	a button
 */

struct XGSTextInitRecord {
	XGSViewInitRecord			v;
	char						text[256];
};

/************************************************************************/
/*																		*/
/*	Standard Classes													*/
/*																		*/
/************************************************************************/

/*	XGStdText
 *
 *		The root standard text object.
 */

class XGStdText : public XGView {
	public:
								XGStdText(XGView *parent, XGArgStream &init, bool f = false);
								XGStdText(XGView *parent, XGSViewInitRecord &init, bool f = false);
		virtual					~XGStdText();
		
		/*
		 *	Text hooks
		 */
		
		virtual void			GetText(char *) = 0;
		virtual void			SetText(char *) = 0;
};

/*	XGStdStaticText
 *
 *		Standard static text control. I do this myself...
 */

class XGStdStaticText : public XGStdText {
	public:
								XGStdStaticText(XGView *parent, XGArgStream &init);
								XGStdStaticText(XGView *parent, XGSTextInitRecord &init);
		virtual					~XGStdStaticText();
		
		/*
		 *	Standard behaviour routines
		 */
		
		void					DoDrawView(Rect);
		
		/*
		 *	Text hooks
		 */
		
		void					GetText(char *);
		void					SetText(char *);

	private:
		void					Init(char *);
	
#if OPT_MACOS == 1
		short					fFont;
		short					fSize;
		XGString				fString;
#endif
};


/*	XGStdEditText
 *
 *		And this is my standard edit control
 */

class XGStdEditText : public XGStdText {
	public:
								XGStdEditText(XGView *parent, XGArgStream &init);
								XGStdEditText(XGView *parent, XGSTextInitRecord &init);
		virtual					~XGStdEditText();
		
		/*
		 *	Standard behaviour routines
		 */
		
		void					DoDrawView(Rect);
		void					DoActivate(bool);
		bool					DoMouseDown(Point,short);
		void					DoSizeView(void);
		void					DoMoveView(void);
		
		void					GainFocus();
		void					LoseFocus();
		
		/*
		 *	Dispatched messages
		 */
		
		long					ReceiveDispatch(long, long, void *);

		/*
		 *	Text selection
		 */
		
		void					SetSelect(short,short);
		void					GetSelect(short *, short *);
		
		/*
		 *	Text Hooks
		 */
		
		void					GetText(char *);
		void					SetText(char *);
		
		/*
		 *	Standard editor modifier hooks
		 */
		
		virtual bool			IsSpecialKey(char);
		virtual void			DrawTextFrame(Rect);
		
	private:
		void					Init(char *);
	
#if OPT_MACOS == 1
		TEHandle				fEdit;
#endif
};


/*	XGStdGroupBox
 *
 *		Standard group box. This creates a group box which draws
 *	a rectangle around the view. On the Macintosh, I duplicate the
 *	functionality in the XGStdStaticText view; in Windows, I call
 *	up the correct view window
 */

class XGStdGroupBox : public XGStdText {
	public:
								XGStdGroupBox(XGView *parent, XGArgStream &init);
								XGStdGroupBox(XGView *parent, XGSTextInitRecord &init);
								~XGStdGroupBox();

		/*
		 *	Group box
		 */
		
		void					DoDrawView(Rect);
		
		/*
		 *	Text hooks
		 */
		
		void					GetText(char *);
		void					SetText(char *);

	private:
		void					Init(char *);
	

#if OPT_MACOS == 1
		short					fFont;
		short					fSize;
		XGString				fString;
#endif
};



#if defined(__MWERKS__)
	#if defined(macintosh)
		#pragma options align=reset
	#endif
	#if defined(__INTEL__)
		#pragma pack(pop)
	#endif
#endif

#endif
